; NAME:
;     extractspec_ps
;
; PURPOSE:
;     Extracts point source spectra from 2-D images.
;       
; CATEGORY:
;     data reduction
;
; CALLING SEQUENCE:
;     result = extractspec_ps(image,var,edgecoeffs,tracecoeffs,norders,naps,$
;                             start,stop,xranges,slith_arc,apradii,apsign,$
;                             BGSTART=bgstart,BGWIDTH=bgwidth,BGORDER=bgorder,$
;                             UPDATE=update,WIDGET_ID=widget_id,CANCEL=cancel
;
; INPUTS:
;     image       - A 2-D image with spectra to be extracted
;     var         - The 2-D variance image
;     edgecoeffs  - Array [degree+1,2,norders] of polynomial coefficients 
;                   which define the edges of the orders.  array[*,0,0]
;                   are the coefficients of the bottom edge of the
;                   first order and array[*,1,0] are the coefficients 
;                   of the top edge of the first order.
;     tracecoeffs - Array [fitdegree+1,naps*norders] of polynomial 
;                   coefficients of the traces of the apertures.
;                   The coefficients should be indexed starting with 
;                   the bottom order and looping through the apertures
;                   in that order.
;     norders     - The number of orders
;     naps        - The number of apertures
;     start       - Start column
;     stop        - Stop column
;     xranges     - An array [2,norders] of pixel positions where the
;                   orders are completely on the array
;     slith_arc   - Slit length in arcsecs.
;     apradii     - Array of aperture radii in arcseconds
;     apsign      - Array of 1s and -1s indicating which apertures
;                   are positive and which are negative (for IR pair
;                   subtraction). 
;
; OUTUTS:
;     Returns an (stop-start+1,naps,norders) array of spectra
; 
; KEYWORD PARAMETERS:   
;     BGSTART   - The radius in arcseconds at which to start the
;                 background region definition (see mkmask_ps)
;     BGWIDTH   - The width of the background region in arcseconds
;                 (see mkmask_ps)
;     BGORDER   - Polynomial fit degree order of the background.  If
;                 omitted, then the background is not subtracted.
;     UPDATE    - If set, the program will launch the Fanning
;                 showprogress widget.
;     WIDGET_ID - If given, a cancel button is added to the Fanning
;                 showprogress routine.  The widget blocks the
;                 WIDGET_ID, and checks for for a user cancel
;                 command.
;     CANCEL    - Set on return if there is a problem
;     
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     mkmask_ps
;     polyfit_1d
;  
; PROCEDURE:
;     At every column within an order, mkmask_ps is called to 
;     create a mask of aperture regions, and background regions.  
;     The background is subtracted if BGORDER is given and then the 
;     flux and variance is summed within defined aperture regions.
;     Set plot1 to see extraction in progress.
;
; REVISION HISTORY:
;     2000-08-24 - Written by M. Cushing, Institute for Astronomy, UH
;     2001-10-04 - Added xranges input
;
function extractspec_ps,image,var,edgecoeffs,tracecoeffs,norders,naps,$
                        start,stop,xranges,slith_arc,apradii,apsign,$
                        BGSTART=bgstart,BGWIDTH=bgwidth,BGORDER=bgorder,$
                        UPDATE=update,WIDGET_ID=widget_id,CANCEL=cancel

plot1  = 0
prange  =[500,550]
if plot1 then window, 9
if plot1 then !p.multi=0

cancel = 0

;  Check parameters

if n_params() lt 12 then begin

    cancel = 1
    return, 1

endif

zparcheck, 'extractspec_ps', image, 1, [2,3,4,5], 2, '2D image'
zparcheck, 'extractspec_ps', var,2,[2,3,4,5],2,'Variance image'
zparcheck, 'extractspec_ps', edgecoeffs, 3, [2,3,4,5], [2,3], 'Edge Coeff'
zparcheck, 'extractspec_ps', tracecoeffs, 4, [2,3,4,5], [1,2], 'Trace Coeff'
zparcheck, 'extractspec_ps', norders, 5, [2,3,4,5], 0, 'Norders'
zparcheck, 'extractspec_ps', naps, 6, [2,3,4,5], 0, 'Naps'
zparcheck, 'extractspec_ps', start, 7, [2,3,4,5], 0, 'Start Column'
zparcheck, 'extractspec_ps', stop, 8, [2,3,4,5], 0, 'Stop Column'
zparcheck, 'extractspec_ps', xranges, 9, [2,3,4,5], [1,2], 'Xranges'
zparcheck, 'extractspec_ps', slith_arc, 10, [2,3,4,5], 0, 'Slit Height'
zparcheck, 'extractspec_ps', apradii, 11, [2,3,4,5], 0, 'Aperture Radius'
zparcheck, 'extractspec_ps', apsign, 12, [2,3,4,5], [0,1], 'Apsign'

;  Get size and define arrays.

s       = size(image)
ncols   = s[1]
nrows   = s[2]

spectra = replicate(!values.f_nan,stop-start+1,2,naps*norders)
x       = findgen(stop-start+1)+start
y       = findgen(nrows)

;  Set up the Fanning showprogress object if requested.

if keyword_set(UPDATE) then begin
    cancelbutton = (n_elements(WIDGET_ID) ne 0) ? 1:0
    progressbar = obj_new('SHOWPROGRESS',widget_id,color=2,$
                          CANCELBUTTON=cancelbutton,$
                          MESSAGE='Extracting Spectra...')
    progressbar -> start

endif

for i = 0, norders-1 do begin

    trace_pix = fltarr(stop-start+1,naps)
    pixtoarc  = fltarr(stop-start+1,2)

    bot = poly(x,edgecoeffs[*,0,i]) 
    top = poly(x,edgecoeffs[*,1,i]) 

    for j = 0, naps-1 do begin

        l = i*naps+j
        trace_pix[*,j] = poly(x,tracecoeffs[*,l])

    endfor
    pixtoarc[*,1] = float(slith_arc) / (top-bot) 
    pixtoarc[*,0] = -1.* (pixtoarc[*,1] * bot)
    
    for j = 0,stop-start do begin

;  Create the slit mask

        if x[j] lt xranges[0,i] or x[j] gt xranges[1,i] then goto, cont2
        
        low       = 0 > bot[j]
        high      = nrows-1 < top[j]
        slity_pix = y[low:high]
        slity_arc = poly(slity_pix,pixtoarc[j,*])
        slitz     = image[x[j],low:high]
        vslitz    = var[x[j],low:high]
        trace_arc = reform(poly(trace_pix[j,*],pixtoarc[j,*]) )

        mask = mkmask_ps(slity_arc,trace_arc,apradii,BGSTART=bgstart,$
                         BGWIDTH=bgwidth,CANCEL=cancel)
        if cancel then return, -1

;  Subtract the background if requested

        if n_elements(BGORDER) ne 0 then begin

            z      = where(mask eq -1)

            if plot1 and x[j] gt prange[0] and x[j] lt prange[1] then $
              print,sqrt(vslitz[z])
            coeff  = poly_fit1d(slity_arc[z],slitz[z],bgorder,$
                                YERR=sqrt(vslitz[z]),/SILENT,VAR=cvar)
            slitz  = temporary(slitz)-poly1d(slity_arc,coeff,cvar,YVAR=yvar)
            vslitz = temporary(vslitz)+yvar

;            coeff  = poly_fit1d(slity_arc[z],slitz[z],bgorder,/SILENT)
;            slitz  = temporary(slitz)-poly1d(slity_arc,coeff)
;            vslitz = temporary(vslitz)

        endif
        if plot1 and x[j] gt prange[0] and x[j] lt prange[1] then begin
;        if plot1 then  begin

            title='Order '+strcompress(i,/re)+', Column '+strcompress(x[j],/re)
            plot, slity_arc,slitz,/xsty,title=title,psym=10,xrange=[0,$
                                                                    slith_arc]
            if n_elements(BGORDER) ne 0 then begin

                z         = where(mask eq -1, count)
                bgmask    = fltarr(n_elements(slity_arc)) + !values.f_nan
                bgmask[z] = slitz[z]
                oplot,slity_arc,bgmask,psym=10,color=2
                
            endif
            z         = where(mask gt 0, count)
            bgmask    = fltarr(n_elements(slity_arc)) + !values.f_nan
            bgmask[z] = slitz[z]
            oplot,slity_arc,bgmask,psym=10,color=3

            for m = 0, naps-1 do begin

                plots,[trace_arc[m],trace_arc[m]],!y.crange,color=4
                plots,[trace_arc[m]-apradii,trace_arc[m]-apradii],$
                  !y.crange,color=3,linestyle=1
                plots,[trace_arc[m]+apradii,trace_arc[m]+apradii],$
                  !y.crange,color=3,linestyle=1
               
            endfor

        endif
        for k = 0, naps-1 do begin

            l = i*naps+k
            z = where(mask gt float(k) and mask le float(k+1))

            if plot1 and x[j] gt prange[0] and x[j] lt prange[1] then begin
;            if plot1 then begin

                if apsign[k] gt 0 then begin

                    print, 'Column ',x[j]
                    print, ' '

                    print, 'Extraction'
                    print, ' '
                    print, trace_arc[k]-apradii,trace_arc[k],trace_arc[k]+$
                      apradii
                    print, ' '
                    vv = vslitz[z]
                    vs = slitz[z]
                    vm = mask[z]
                    va = slity_arc[z]
                    vp = slity_pix[z]
                    for q = 0,n_elements(vs)-1 do print,vp[q],va[q],vs[q],$
                      vv[q],vm[q]-float(k),(vm[q]-float(k))*vs[q],$
                      vs[q]/sqrt(vv[q])

                endif
                
            endif

;  Extract the spectra

            spectra[j,0,l] =float(apsign[k])*total(slitz[z]*(mask[z]-float(k)))
            spectra[j,1,l] =total(vslitz[z]*(mask[z]-float(k))^2)

            if plot1 and x[j] gt prange[0] and x[j] lt prange[1] then begin
;            if plot1 then begin

                print, 'TOTAL = ',spectra[j,0,l]
                print, 'TOTAL VAR = ',spectra[j,1,l]
                re = ' '
                read, re

            endif

        endfor
        cont2:

    endfor

    if keyword_set(UPDATE) then begin

        if cancelbutton then begin

            cancel = progressBar->CheckCancel()
            if cancel then begin

                progressBar->Destroy
                obj_destroy, progressbar
                cancel = 1
                return, -1

            endif

        endif
        percent = (i+1)*(100./float(norders))
        progressbar->update, percent

    endif

endfor

if keyword_set(UPDATE) then begin

    progressbar-> destroy
    obj_destroy, progressbar

endif

return, spectra    

end


