; NAME:
;     findlines
;
; PURPOSE:
;     To identify arc line positions given the guesses of their positions.
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     result = findlines(x,y,xguess,SIGMA=sigma,GOODBAD=goodbad,PLOT=plot)
;
; INPUTS:
;     x      - Array of indenpendent values
;     y      - Array of dependent values (the spectrum)
;     xguess - An array of guess positions (in units of x) of the lines
;      sigma  - A scalar giving the theoretical standard deviation of
;              the lines in units of x.
;
; OUTUTS:
;     Returns an array of the line positions in units of x.
;
; KEYWORD PARAMETERS:    
;     GOODBAD - An array of 1s (good) and 0s (bad) to identify which
;               lines where identified
;     PLOT    - Set to plot the fits
;     WLINES  - Given an array of lines associated with the xguess
;               positions the program will label the plots with the
;               wavelength. 
;     CANCEL  - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     Requires the Markwardt Fitting Library (Find at Astronomy User's Library)
;
; PROCEDURE:
;     Each line is fit with a gaussian to determine the line center.
;
; MODIFICATION HISTORY:
;     Written 2/10/99 by M. Cushing, Institute for Astronomy, UH
;
function findlines,x,y,xguess,sigma,GOODBAD=goodbad,PLOT=plot,WLINES=wlines,$
                   CANCEL=cancel

cancel = 0

;  Check parameters 

if n_params() lt 4 then begin
    
    print, 'Syntax - result = findlines(x,y,xguess,sigma,GOODBAD=goodbad,$'
    print, '                            PLOT=plot,WLINES=wlines,CANCEL=cancel)'
    cancel = 1
    return, -1

endif
zparcheck, 'findlines', x, 1, [2,3,4,5], 1, 'X'
zparcheck, 'findlines', y, 2, [2,3,4,5], 1, 'Y'
zparcheck, 'findlines', sigma, 3, [2,3,4,5], 0, 'sigma'

;  Get basic info

xmax   = max(x,min=xmin)
nx     = n_elements(x)
nlines = n_elements(xguess)
hwin   = fix(sigma*(10.))

; Initialize the goodbad and lines array.

goodbad = intarr(nlines)
lines   = fltarr(nlines)
y       = float(y)

if plot then begin

    window,/free
    win_idx = !d.window
    plot,x,y,/xsty,/ysty
    re = ' '
    read, re

endif

;  Set up Gaussian

expr        = 'p[0] + p[1]*x + gaussian(x,p[2:4])'
;pi          = replicate({fixed:0, limited:[0,0], limits:[0,0]},5)

for i = 0, nlines-1 do begin

;  Extract the line to be fit, ignoring lines with NaNs (usually the edge.)

    tabinv, x,xguess[i],idx
    xwin  = float( x[(0 > (round(idx)-hwin)):(nx-1 < (round(idx)+hwin))] )
    ywin  =        y[(0 > (round(idx)-hwin)):(nx-1 < (round(idx)+hwin))] 
    nan   = where(finite(ywin) eq 0,count)
    if count ne 0 then goto, cont

;  Fit the line.

    tabinv,xwin,xguess[i],idx
    start =float( [0.,0.,ywin[round(idx)],xguess[i],sigma] )
    fit = mpfitexpr(expr,xwin,ywin,dummy,start,PARINFO=pi,/QUIET,/NOCOVAR)
    
    lines[i] = fit[3]
    if fit[3] le xguess[i]+5 and fit[3] ge xguess[i]-5 and fit[2] gt 0 and $
      fit[4] gt sigma/2. then begin

        goodbad[i]  = 1

    endif
    if plot then begin

        if goodbad[i] eq 1 then title='!5GOOD' else title='!5BAD'
        if n_elements(WLINES) ne 0 then title=title+' '+strtrim(wlines[i],2)
        plot,xwin,ywin,/xsty,psym=10,title=title
        plots,[xguess[i],xguess[i]],!y.crange,color=6
        if goodbad[i] eq 0 then print, fit[3]
;        print, fit
        f = ( findgen(n_elements(xwin)*10.) )/10.  + min(xwin)
        oplot,f,poly(f,fit[0:1])+gaussian(f,fit[2:4]),color=2,psym=10
        oplot,xwin,poly(xwin,fit[0:1])+gaussian(xwin,fit[2:4]),color=3,psym=10
        re = ' '
        read, re
        
    endif
    cont:

endfor

if plot then wdelete, win_idx
return, lines

end







