; NAME:
;     findorders
;
; PURPOSE:
;     To determine the position of the order(s) in an spectral image.
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     findorders,image,guesspos,start,stop,step,slith_pix,degree,halfwin,$
;                edgecoeffs,xranges,WID=wid,PLOTGUESS=plotguess,CANCEL=cancel)
;
; INPUTS:
;     image     - Flat field image
;     guesspos  - An array [2,norders] of positions x,y located
;                 within each order.  The positions should be near the 
;                 center of the order and be located in a region of 
;                 large flux.
;     start     - Start column
;     stop      - Stop column
;     step      - Step size in the dispersion direction 
;     slith_pix - Approximate slit length in pixels
;     degree    - Polynomial fit degree for the edges of the orders
;     halfwin   - Half the window used to compute the centroid (see below).
;
; OUTUTS:
;     edgecoeffs - Array [degree+1,2,norders] of polynomial coefficients 
;                  which define the edges of the orders.  array[*,0,0]
;                  are the coefficients of the bottom edge of the
;                  first order and array[*,1,0] are the coefficients 
;                  of the top edge of the first order.
;     xranges    - An array [2,norders] of pixel positions where the
;                  orders are completely on the array
;
; KEYWORD PARAMETERS:    
;     WID       - Window ID of a window to plot the results
;     PLOTGUESS - Set to plot the guess solution positions
;     CANCEL    - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     robustpoly1d
;
; PROCEDURE:
;     The IDL sobel function is used to enhance the edges of the
;     orders.  Within an order, and at a particular column, the
;     positions above and below the guess position at which the flux
;     falls to 0.75 the value at the guess position is determined.  
;     Centroids are computed at these positions on the Sobeled image
;     to determine the location of the edge of the order.  As the
;     program steps away from the guess position, the new guess
;     position is estimated using the previously determined edges.
;     After the entire array has been checked, the results are fit
;     with a robust least-squares polynomial of degree degree.
;
; MODIFICATION HISTORY:
;     2001-05-01 - Written by Michael Cushing, Institute for Astronomy, UH
;     2001-10-02 - Added xrange input keyword and changed to a procedure
;     2001-11-06 - Added the PLOTGUESS keyword
;
pro findorders,image,guesspos,start,stop,step,slith_pix,degree,halfwin,$
               edgecoeffs,xranges,WID=wid,PLOTGUESS=plotguess,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 8 then begin
    
    print, 'Syntax - findorders,image,guesspos,start,stop,step,slith_pix,$'
    print, '         degree,halfwin,edgecoeffs,xranges,WID=wid,$'
    print, '         PLOTGUESS=plotguess,CANCEL=cancel'
    cancel = 1
    return

endif

zparcheck, 'findorders', image, 1, [2,3,4,5], 2, 'Image' 
zparcheck, 'findorders', guesspos, 2, [2,3,4,5], [1,2],'Guess Positions' 
zparcheck, 'findorders', start, 3, [2,3,4,5],0, 'Start Column' 
zparcheck, 'findorders', stop, 4, [2,3,4,5],0, 'Stop Column' 
zparcheck, 'findorders', slith_pix, 5, [2,3,4,5],0, 'Slit Height (Pixels)' 
zparcheck, 'findorders', degree, 6, [2,3,4,5],0, 'Polynomial degree' 
zparcheck, 'findorders', halfwin, 7, [2,3,4,5],0, 'Halfwin' 

frac = 0.85

if n_elements(WID) ne 0 then wset, wid

;  Determine useful info

s       = size(guesspos)
norders = (s[0] eq 1) ? 1:s[2]
s       = size(image)
ncols   = s[1]
nrows   = s[2]

;  Set up parameters.

starts = start+step-1  
stops  = stop -step+1 
secol  = findgen(fix((stops-starts)/step)+1)*step + starts
nscol  = n_elements(secol) 
edges  = replicate(!values.f_nan,nscol,2,norders)
row    = findgen(nrows)
cen    = fltarr(nscol)+!values.f_nan

;  Smooth, scale, and sobel the image

max    = max(image)
rimage = roberts( median( (image*10000./max), 4) )

for i = 0, norders-1 do begin

    plots,[guesspos[0,i],guesspos[0,i]],[guesspos[1,i],guesspos[1,i]],$
      color=2,psym=2,symsize=1.5
    cen = fltarr(nscol)+!values.f_nan

    tabinv,secol,guesspos[0,i],idx
    ridx = round(idx)
    cen[(ridx-1):(ridx+1)] = guesspos[1,i]

    for j = 0, idx do begin
    
        k = ridx-j

        col  = reform( image[secol[k],*] )
        rcol = reform( rimage[secol[k],*] )
        
        coeff = poly_fit1d(secol,cen,2,/SILENT)
        y_guess = poly(secol[k],coeff)
        if y_guess gt nrows or y_guess lt 0 then goto, cont1
        if keyword_set(PLOTGUESS) then plots,[secol[k],secol[k]],$
          [y_guess,y_guess],color=2,psym=3,symsize=1.5
        
        peak_above = where(col lt frac*col[y_guess] and row gt y_guess,count)
        if count eq 0 then goto, cont1
        peak_below = where(col lt frac*col[y_guess] and row lt y_guess,count)
        if count eq 0 then goto, cont1
        guessy_top = peak_above[0]
        guessy_bot =peak_below[n_elements(peak_below)-1]

;  Fit the center-of-mass around the peaks.

        com = replicate(!values.f_nan,2)

        bot_y  = 1 > (guessy_bot-halfwin)
        top_y  = (guessy_bot+halfwin) < (nrows-1)
        y      = row[bot_y:top_y]
        z      = rcol[bot_y:top_y]            
        com[0] = total(y*z)/total(z)
        
        bot_y  = 1 > (guessy_top-halfwin)
        top_y  = (guessy_top+halfwin) < (nrows-1)
        y      = row[bot_y:top_y]
        z      = rcol[bot_y:top_y]
        com[1] = total(y*z)/total(z)
            
        if abs(com[0]-com[1]) gt slith_pix - 10 and $
          abs(com[0]-com[1]) lt slith_pix + 10 then begin
            
            edges[k,*,i] = com
            cen[k] = total(com)/2.
            if n_elements(wid) ne 0 then begin
                
                plots,[secol[k],secol[k]],[com[0],com[0]],$
                  psym=1, color=3
                plots,[secol[k],secol[k]],[com[1],com[1]],$
                  psym=1, color=3
                
            endif        
            
        endif
    
        cont1:
        
    endfor

    for j = ridx+1,nscol-1 do begin
    
        k = j

        col  = reform( image[secol[k],*] )
        rcol = reform( rimage[secol[k],*] )
        
        coeff = poly_fit1d(secol,cen,2,/SILENT)
        y_guess = poly(secol[k],coeff)
        if y_guess gt nrows or y_guess lt 0 then goto, cont2
        if keyword_set(PLOTGUESS) then plots,[secol[k],secol[k]],$
          [y_guess,y_guess],color=2,psym=3,symsize=1.5
        
        peak_above = where(col lt frac*col[y_guess] and row gt y_guess,count)
        if count eq 0 then goto, cont2
        peak_below = where(col lt frac*col[y_guess] and row lt y_guess,count)
        if count eq 0 then goto, cont2
        guessy_top = peak_above[0]
        guessy_bot =peak_below[n_elements(peak_below)-1]

;  Fit the center-of-mass around the peaks.

        com = replicate(!values.f_nan,2)

        bot_y  = 1 > (guessy_bot-halfwin)
        top_y  = (guessy_bot+halfwin) < (nrows-1)
        y      = row[bot_y:top_y]
        z      = rcol[bot_y:top_y]            
        com[0] = total(y*z)/total(z)
        
        bot_y  = 1 > (guessy_top-halfwin)
        top_y  = (guessy_top+halfwin) < (nrows-1)
        y      = row[bot_y:top_y]
        z      = rcol[bot_y:top_y]
        com[1] = total(y*z)/total(z)
            
        if abs(com[0]-com[1]) gt slith_pix - 10 and $
          abs(com[0]-com[1]) lt slith_pix + 10 then begin
            
            edges[k,*,i] = com
            cen[k] = total(com)/2.
            if n_elements(wid) ne 0 then begin
                
                plots,[secol[k],secol[k]],[com[0],com[0]],$
                  psym=1, color=3
                plots,[secol[k],secol[k]],[com[1],com[1]],$
                  psym=1, color=3
                
            endif        
            
        endif
    
        cont2:
        
    endfor

endfor

;  Fit the results

edgecoeffs = dblarr(degree+1,2,norders)
x          = findgen(stop-start+1)+start

for i = 0,norders-1 do begin

    for j = 0,1 do begin

            coeff = robustpoly1d(secol,edges[*,j,i],degree,3,0.01,$
                                 OGOODBAD=goodbad,/SILENT,/GAUSSJ)
            edgecoeffs[*,j,i] = coeff

        if n_elements(WID) ne 0 then begin

            z = where(finite(edges[*,0,i]) eq 1)
            oplot, secol[z],poly(secol[z],coeff),color=2
            z = where(goodbad eq 0,count)
            if count ne 0 then oplot, secol[z],edges[z,j,i],color=4,psym=1

        endif

    endfor
    
endfor

;  Find the xrange for each order

xranges      = intarr(2,norders)
xranges[0,*] = start
xranges[1,*] = stop

for i = 0, norders-1 do begin

    z = where(finite(edges[*,0,i]) eq 1)
    colmin = min(secol[z],max=colmax)

;  Move to the left

    while colmin gt start do begin
        
        y_bot = poly(colmin,edgecoeffs[*,0,i])
        y_top = poly(colmin,edgecoeffs[*,1,i])
        if y_bot[0] lt 0. or y_top[0] gt nrows-1 then begin

            xranges[0,i] = colmin
            goto, out1
        
        endif else colmin = colmin-1

    endwhile
    out1:

;  Now right

    while colmax lt stop do begin

        y_bot = poly(colmax,edgecoeffs[*,0,i])
        y_top = poly(colmax,edgecoeffs[*,1,i])
        if y_bot lt 0 or y_top gt nrows-1 then begin

            xranges[1,i] = colmax
            goto, out2

        endif else colmax = colmax+1

    endwhile
    out2:

endfor

end
