; NAME:
;     findoutliers
;
; PURPOSE:
;     To determine the outliers in a distribution using the median
;     and median absolute deviation.
;       
; CATEGORY:
;     statistics
;
; CALLING SEQUENCE:
;     result = findoutliers(data,thresh,CANCEL=cancel)
;
; INPUTS:
;     data   - Input data
;     thresh - The sigma threshold for the robust algorithm
;
; OUTUTS:
;     Returns a mask of same size as data where 1=good,0=bad.
;
; KEYWORD PARAMETERS:    
;     CANCEL - Set on return if there is an problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library.
;
; PROCEDURE:
;     Computes the median and the median absolute deviation 
;     1.482*median(|x_i-x_med|)
;     (http://www.soest.hawaii.edu/wessel/courses/gg313/DA_book/node37.html)
;     of the data and identifies data values as outliers if 
;     |x_i - x_med|/MAD > thresh where x_med is the median and MAD is 
;     the median absolute deviation.
;
; REVISION HISTORY:
;     2001-04-22 - written by M. Cushing, Institute for Astronomy, UH
;
function findoutliers,data,thresh,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 2 then begin
    
    print, 'Syntax - result = findoutliers(data,thresh,CANCEL=cancel)'
    cancel = 1
    return,1

endif
zparcheck, 'findoutliers',data, 1, [2,3,4,5], [1,2,3], 'Data' 
zparcheck, 'findoutliers',thresh, 2, [2,3,4,5], 0, 'Sigma Threshold' 

mask = intarr(n_elements(data))+1

med = median(data,/EVEN)
mad = 1.482*median( abs(data-med), /EVEN)
z   = where(abs( (data-med)/mad ) gt thresh,count)

if count ne 0 then mask[z] = 0.

return, mask


end

