; NAME:
;     meancomb
;
; PURPOSE:
;     To combine a data set using a (weighted) mean.
;       
; CATEGORY:
;     data reduction
;
; CALLING SEQUENCE:
;     meancomb,data,mean,var,DATAVAR=datavar,UPDATE=update,$
;              WIDGET_ID=widget_id,CANCEL=cancel
;
; INPUTS:
;     data  - Data array (1, 2 or 3D).
;
; OUTUTS:
;     mean - The mean array.
;     mvar - The "variance of the mean", sigma_mu^2, array.
;
; KEYWORD PARAMETERS:    
;     DATAVAR   - The variances of the data points.  If given, a
;                 weighted mean is performed.
;     UPDATE    - If set, the program will launch the Fanning
;                 showprogress widget.
;     WIDGET_ID - If given, a cancel button is added to the Fanning
;                 showprogress routine.  The widget blocks the
;                 WIDGET_ID, and checks for for a user cancel
;                 command.
;     CANCEL    - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library.
;     Fanning ShowProgress Object
;
; PROCEDURE:
;     This routine will combine either 1, 2 or 3-D data using either
;     a straight mean or weighted mean.  If DATAVAR are not given, 
;     then a straight mean, <x>, and variance on the mean, 
;     sigma_mu^2 = sigma^2/N are computed.  If DATAVAR are given, 
;     then a weighted mean and corresponding variance on the mean are
;     computed. 
;
; REVISION HISTORY:
;     2001-04-22 - Written by M. Cushing, Institute for Astronomy, UH
;
pro meancomb,data,mean,mvar,DATAVAR=datavar,UPDATE=update,$
             WIDGET_ID=WIDGET_ID,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 1 then begin
    
    print, 'Syntax - meancomb,data,mean,mvar,DATAVAR=datavar,$'
    print, '                  UPDATE=update,WIDGET_ID=widget_id,CANCEL=cancel'
    cancel = 1
    return

endif

zparcheck, 'meancomb',data, 1, [2,3,4,5], [1,2,3], 'Data array' 

;  Check for weights.

if n_elements(DATAVAR) ne 0 then weighted = 1 else weighted=0

;  Get size of data array.

s = size(data)

case s[0] of 

    1: narr = n_elements(data)

    2: begin

        ncols = s[1]
        narr  = s[2]

    end
    
    3: begin

        ncols = s[1]
        nrows = s[2]
        narr  = s[3]
        mean  = dblarr(ncols,nrows,/NOZERO)
        mvar  = dblarr(ncols,nrows,/NOZERO)

    end

endcase

;  Perform combination.

if s[0] eq 1 then begin

    if weighted then begin

        mvar = 1./total(1./datavar,/DOUBLE)
        mean = total(data/datavar,/DOUBLE)*mvar

    endif else begin

        mean = total(data,/DOUBLE)/narr
        mvar = ( total(double(data)^2)-narr*mean^2 )/(narr-1)/narr

    endelse

endif

if s[0] eq 2 then begin

    if weighted then begin

        mvar = 1./total(1./datavar,2,/DOUBLE)
        mean = total(data/datavar,2,/DOUBLE)*mvar

    endif else begin

        mean = total(data,2,/DOUBLE)/narr
        mvar = ( total(double(data)^2,2)-narr*mean^2 )/(narr-1)/narr

    endelse

endif

if s[0] eq 3 then begin

    if weighted then begin

;  Set up the Fanning showprogress object if requested.

        if keyword_set(UPDATE) then begin
            cancelbutton = (n_elements(WIDGET_ID) ne 0) ? 1:0
            progressbar = obj_new('SHOWPROGRESS',widget_id,color=2,$
                                  CANCELBUTTON=cancelbutton,title='Wmeancomb',$
                                  MESSAGE='Averaging the images...')
            progressbar -> start
            
        endif
        
        for i = 0,ncols-1 do begin
            
            for j = 0, nrows-1 do begin

                mean[i,j] = total(data[i,j,*]/datavar[i,j,*],/DOUBLE)
                mvar[i,j] = 1./total(1./datavar[i,j,*],/DOUBLE)

            endfor
            if keyword_set(UPDATE) then begin
                
                if cancelbutton then begin
                    
                    cancel = progressBar->CheckCancel()
                    if cancel then begin
                        
                        progressBar->Destroy
                        obj_destroy, progressbar
                        cancel = 1
                        return
                        
                    endif
                    
                endif
                percent = (i+1)*(100./float(ncols))
                progressbar->update, percent
                
            endif
            
        endfor
        mean = temporary(mean)*mvar

        if keyword_set(UPDATE) then begin
            
            progressbar-> destroy
            obj_destroy, progressbar
            
        endif

    endif else begin

        mean = total(data,3,/DOUBLE)/narr
        mvar = ( total(double(data)^2,3)-narr*mean^2 )/((narr-1)*narr)

    endelse

endif

mean = float(mean)
mvar = float(mvar)


end




