; NAME:
;     medcomb
;
; PURPOSE:
;     To combine a data set using a median.
;       
; CATEGORY:
;     data reduction
;
; CALLING SEQUENCE:
;     medcomb,data,med,var,UPDATE=update,WIDGET_ID=widget_id,CANCEL=cancel
;
; INPUTS:
;     data    - Data array (1, 2 or 3D).
;
; OUTUTS:
;     med - The median data
;     var - An estimate of the variance array (MAD^2 / N).
;
; KEYWORD PARAMETERS:    
;     UPDATE    - If set, the program will launch the Fanning
;                 showprogress widget.
;     WIDGET_ID - If given, a cancel button is added to the Fanning
;                 showprogress routine.  The widget blocks the
;                 WIDGET_ID, and checks for for a user cancel
;                 command.
;     CANCEL    - Set on return if there is an error. 
;
; REVISION HISTORY:
;     2001-04-22 - written by M. Cushing, Institute for Astronomy, UH
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library.
;
; PROCEDURE:
;     Combines the data together using a median.  An estimate of the 
;     error is is given by computing the Median Absolute Deviation 
;     (http://www.soest.hawaii.edu/wessel/courses/gg313/DA_book/node37.html)
;     and then med_var = MAD^2/N
;
pro medcomb,data,med,var,UPDATE=update,WIDGET_ID=widget_id,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 1 then begin
    
    print, 'Syntax - mecomb,data,med,war,UPDATE=update,WIDGET_ID=widget_id,$'
    print, '                CANCEL=cancel'
    cancel = 1
    return

endif

zparcheck, 'medcomb',data, 1, [2,3,4,5], [1,2,3], 'Data array' 

s       = size(data)

case s[0] of

    1: narr = n_elements(data)

    2: begin

        ncols = s[1]
        narr  = s[2]
        med   = fltarr(ncols,/NOZERO)
        var   = fltarr(ncols,/NOZERO)

    end

    3: begin

        ncols = s[1]
        nrows = s[2]
        narr  = s[3]
        med   = fltarr(ncols,nrows,/NOZERO)
        var   = fltarr(ncols,nrows,/NOZERO)

    end

endcase

;  Perform combination.

if s[0] eq 1 then begin

    med = median(data,/EVEN)
    var = (1.482*median( abs(data-med), /EVEN))^2

endif else begin

;  Set up the Fanning showprogress object if requested.

    if keyword_set(UPDATE) then begin
        
        cancelbutton = (n_elements(WIDGET_ID) ne 0) ? 1:0
        progressbar = obj_new('SHOWPROGRESS',widget_id,color=2,$
                              CANCELBUTTON=cancelbutton,title='Medcomb',$
                              MESSAGE='Median combining the images...')
        progressbar -> start
        
    endif
    
    for i = 0, ncols-1 do begin
        
        if s[0] eq 2 then begin
            
            med[i] = median(data[i,*],/EVEN)
            var[i] = (1.482*median( abs(data[i,*]-med[i]), /EVEN))^2
            
        endif else begin
            
            for j = 0, nrows-1 do begin
                
                med[i,j] = median(data[i,j,*], /EVEN)
                var[i,j] = (1.482*median( abs(data[i,j,*]-med[i,j]), /EVEN))^2
;            if var[i,j] eq 0 then begin
                
;                print, i,j
;                print, reform(data[i,j,*])
;                re = ' '
;                read, re
                
;            endif
            endfor
            
        endelse
        if keyword_set(UPDATE) then begin
            
            if cancelbutton then begin
                
                cancel = progressBar->CheckCancel()
                if cancel then begin
                    
                    progressBar->Destroy
                    obj_destroy, progressbar
                    cancel = 1
                    return
                    
                endif
                
            endif
            percent = (i+1)*(100./float(ncols))
            progressbar->update, percent
            
        endif
        
    endfor

endelse
z = where(var eq 0,count)
if count ne 0 then var[z] = 1.
var = temporary(var)/float(narr)

if keyword_set(UPDATE) then begin

    progressbar-> destroy
    obj_destroy, progressbar

endif

end














