; NAME:
;     mkfullpath
;
; PURPOSE:
;     To construct a fullpath name for files.
;
; CATEGORY:
;     Data Input
;
; CALLING SEQUENCE:
;     result = mkfullpath(path,files,INDEX=index,FILENAME=filename,NI=ni,$
;                         PREFIX=prefix,SUFFIX=suffix,EXIST=exist,$
;                         WIDGET_ID=widget_id,CANCEL=cancel)
;
; INPUTS:
;     path  - The desired path
;     files - String of either filenames or indices
; 
; OUTUTS:
;     Returns an array of the fullpaths of the files.
;
; KEYWORD PARAMETERS:    
;     INDEX     - Set to construct the full paths if files are indices.  If
;                 set, then NI and PREFIX must also be defined.
;     FILENAME  - Set to construct the full paths if files are file names.
;     NI        - Number of integer positions, i.e. i4.4
;     PREFIX    - The prefix of the file names in index mode
;     SUFFIX    - The suffix of the file names in index mode 
;     EXIST     - Set to check to make sure the files exist
;     WIDGET_ID - If given, an pop-up error message will appear over
;                 the widget. 
;     CANCEL    - Will be set on return if there is a problem.
;
; PROCEDURES CALLED:
;     Required >= IDL 5.3 because of the string procedures.
;
;  PROCEDURE: 
;     Concatinates the path and file where file = prefix+index+suffix
;
; MODIFICATION HISTORY:
;     2000-08-16 - Written by M. Cushing, Institute for Astronomy, UH
;
function mkfullpath,path,files,INDEX=index,FILENAME=filename,NI=ni,$
                    PREFIX=prefix,SUFFIX=suffix,EXIST=exist,$
                    WIDGET_ID=widget_id,CANCEL=cancel
;
cancel = 0

;  Check parameters

if n_params() lt 2 then begin
    
    cancel = 1
    print, 'Syntax -  result = mkfullpath(path,files,INDEX=index,$'
    print, '                              FILENAME=filename,NI=ni,$'
    print, '                              PREFIX=prefix,SUFFIX=suffix,$'
    print, '                              EXIST=exist,$'
    print, '                              WIDGET_ID=widget_id,CANCEL=cancel)'
    return, -1

endif

zparcheck, 'mkfullpath', path, 1,7,0, 'Path Name' 
zparcheck, 'mkfullpath', files, 2,7,[0,1], 'File Names' 

if n_elements(SUFFIX) eq 0 then suffix = ''

widget = (n_elements(WIDGET_ID) ne 0) ? 1:0

;  Break up the files string.

if keyword_set(INDEX) then begin

;  Check to make the indices are less than NI.

    z    = where(files gt (long(10)^ni)-1, count)
    if count gt 0 then begin
        
        cancel = 1
        mess = 'Image numbers greater than '+ $
          strcompress((10^ni)-1,/re)+' not allowed.'
        if widget then ok = dialog_message(mess,/error,dialog_parent=widget_id)
        if not widget then print, mess
        return, -1
        
    endif
    
;  Now build the paths string array.

    for i = 0, n_elements(files) - 1 do begin
        
        fmt= '(i'+strcompress(ni,/re)+'.'+strcompress(ni,/re)+')'
        name = strcompress( path+prefix+string(files[i],format=fmt)+suffix)
        if keyword_set(EXIST) then begin

            tmp = findfile(strtrim(name,2),COUNT=count)
            if count eq 0 then begin
                
                cancel = 1
                mess = 'Image '+name+' does not exist.'
                if widget then ok = dialog_message(mess,/error,$
                                                   dialog_parent=widget_id)
                if not widget then print, mess
                return, -1
                
            endif
            name = tmp

        endif
        fullpaths = (i eq 0) ? name:[fullpaths,name]
        
    endfor

endif
if keyword_set(FILENAME) then begin

;  Build the fname string array.
    
    for i = 0 ,n_elements(files) - 1 do begin
        
        name = path+files[i]
        fullpaths = (i eq 0) ? name:[fullpaths,name]

    endfor

endif

;  Check to see if the files exist if requested.
    
if keyword_set(EXIST) then begin

    nimages = n_elements(fullpaths) 
    for i = 0, nimages - 1 do begin
        
        file = cfile(fullpaths[i],WIDGET_ID=widget_id,CANCEL=cancel)
        if cancel then return, -1

    endfor

endif

return, fullpaths


end
