; NAME:
;     normspecflat
;
; PURPOSE:
;     To normalize a spectral flat field image
;       
; CATEGORY:
;     Data Reduction
;
; CALLING SEQUENCE:
;     result = normspecflat(image,edgecoeffs,norders,xranges,slith_arc,$
;                           sorder,sgwindow,sgdeg,ybuffer,RMS=rms,$
;                           UPDATE=update,WIDGEt_ID=widget_id,CANCEL=cancel)
;
; INPUTS:
;     image      - A spectral flat field image
;     edgecoeffs - Array [degree+1,2,norders] of polynomial coefficients 
;                  which define the edges of the orders.  array[*,0,0]
;                  are the coefficients of the bottom edge of the
;                  first order and array[*,1,0] are the coefficients 
;                  of the top edge of the first order
;     norders    - The number of orders
;     xranges    - An array [2,norders] of pixel positions where the
;                  orders are completely on the array
;     slith_arc  - The length of the slit in arcseconds
;     sorder     - The polynomial degree to fit the spatial order
;     sgwindow   - Width of the Savitzky-Golay smoothing window in pixels
;     sgdeg      - Degree of the Savitzky-Golay polynomail
;     ybuffer    - Number of pixels to move inside of the edgecoeffs.
;                  The edges can cause contamination of the dispersion profile.
;
; OUTUTS:
;     Returns a normalized flat field image
;
; KEYWORD PARAMETERS:    
;     UPDATE    - If set, the program will launch the Fanning
;                 showprogress widget.
;     WIDGET_ID - If given, a cancel button is added to the Fanning
;                 showprogress routine.  The widget blocks the
;                 WIDGET_ID, and checks for for a user cancel
;                 command.
;     CANCEL    - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     robustpoly1d
;
; PROCEDURE:
;     Collapses each order in the dispersion dimension and performs a
;     robust Savitzky-Golay smoothing.  This profile is then divided 
;     out.  Each order is then collapsed in the spatial dimension and 
;     fit with a polynomial of degree sorder.  This result is divided out.
;
; REVISION HISTORY:
;     2000       - Written by M. Cushing, Institute for Astronomy
;     2001-10-04 - Removed the start and stop input and added the
;                  xrange input 
;
function normspecflat,image,edgecoeffs,norders,xranges,slith_arc,sorder,$
                      sgwindow,sgdeg,ybuffer,RMS=rms,UPDATE=update,$
                      WIDGEt_ID=widget_id,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 9 then begin
    
    print, 'Syntax - result = normspecflat(image,edgecoeffs,norders,xranges,$'
    print, '                               slith_arc,sorder,sgwindow,$'
    print, '                               sgdeg,ybuffer,RMS=rms,$'
    print, '                               UPDATE=update,WIDGET_ID=widget_id,$'
    print, '                               CANCEL=cancel'

    cancel = 1
    return, -1

endif
zparcheck, 'normspecflat', image, 1, [2,3,4,5], 2, 'Image'
zparcheck, 'normspecflat', edgecoeffs, 2, [2,3,4,5], [2,3], 'Edgecoeffs'
zparcheck, 'normspecflat', norders, 3, [2,3,4,5], 0, 'Norders'
zparcheck, 'normspecflat', xranges, 4, [2,3,4,5], [1,2], 'Xranges'
zparcheck, 'normspecflat', slith_arc, 5, [2,3,4,5], 0, 'Slith_arc'
zparcheck, 'normspecflat', sorder, 6, [2,3,4,5], 0, 'Sorder'
zparcheck, 'normspecflat', sgwindow, 7, [2,3,4,5], 0, 'SGwindow'
zparcheck, 'normspecflat', sgdeg, 8, [2,3,4,5], 0, 'SGdegree'
zparcheck, 'normspecflat', ybuffer, 9, [2,3,4,5], 0, 'Ybuffer'

;  Determine the size of the image, and number of orders

s       = size(image)
ncols   = s[1]
nrows   = s[2]

;  Initialize two arrays.

y    = findgen(nrows)
flat = fltarr(ncols,nrows)+1.
surf = fltarr(ncols,nrows)+1.

;  Set up the Fanning showprogress object if requested.

if keyword_set(UPDATE) then begin

    cancelbutton = (n_elements(WIDGET_ID) ne 0) ? 1:0
    progressbar = obj_new('SHOWPROGRESS',widget_id,color=2,$
                          CANCELBUTTON=cancelbutton,title='Spextool',$
                          MESSAGE='Normalizing the Flat Field...')
    progressbar -> start

endif

;  Normalize each order.

tmp  = fltarr(ncols,nrows)+1.
rms  = fltarr(norders)

for i = 0,norders-1 do begin

    start = xranges[0,i]
    stop  = xranges[1,i]
    x     = findgen(stop-start+1)+start

    disp = fltarr(stop-start+1)+!values.f_nan
    mask = intarr(ncols,nrows)

    botedge = poly1d(x,edgecoeffs[*,0,i])
    topedge = poly1d(x,edgecoeffs[*,1,i])

;  Get dispersion profile and normalize the order.

    for j = 0,stop-start do begin
        
        if botedge[j] + ybuffer lt -0.5 then goto, cont1
        if topedge[j] - ybuffer gt nrows-0.5 then goto, cont1
                
        y_bot = round( 0 > (botedge[j]+ybuffer) )
        y_top = round( (nrows-1) < (topedge[j]-ybuffer) )
        disp[j] = median(image[x[j],y_bot:y_top],/EVEN)
        mask[x[j],y_bot:y_top] = 1

        cont1:

    endfor

    result = robustsg(x,disp,sgwindow,3,0.01,OGOODBAD=ogoodbad,DEGREE=sgdeg)

;  Now normalize the order.
    
    for j = 0, stop-start do begin
        
        if botedge[j] + ybuffer lt -0.5 then goto, cont2
        if topedge[j] - ybuffer gt nrows-0.5 then goto, cont2
        
        y_bot = round( 0 > (botedge[j]+ybuffer) )
        y_top = round( (nrows-1) < (topedge[j]-ybuffer) )
        if finite(result[j,1]) ne 1 then goto, cont2
        
        tmp[x[j],y_bot:y_top] = image[x[j],y_bot:y_top]/result[j,1]

        cont2:

    endfor

;  Determine the normalized spatial profile.

    dif  = topedge-botedge
    syy  = 0
    szz  = 0
    spixtoarc = fltarr(stop-start+1,2)
    for j = 0, stop-start do begin

        if botedge[j]+ybuffer lt -0.5  then goto, cont3
        if topedge[j]-ybuffer gt nrows-0.5 then goto, cont3
        
        pixtoarc    = fltarr(2)            
        pixtoarc[1] = float(slith_arc)/float(dif[j])
        pixtoarc[0] = -1.* pixtoarc[1] * botedge[j]
        
        spixtoarc[j,*] = pixtoarc
        
        y_bot = round( 0 > (botedge[j] +ybuffer) )
        y_top = round( (nrows-1) < (topedge[j] -ybuffer) )

        yy    = y[y_bot:y_top]

        yy_arc = poly(yy,pixtoarc)
        zz     = reform(tmp[x[j],y_bot:y_top] )
        
        syy = [syy,yy_arc]
        szz = [szz,zz]
       
        cont3:
       
    endfor

    syy = syy[1:*]
    szz = szz[1:*]

    z = sort(syy)
    syy = syy[z]
    szz = szz[z]

    coeff =robustpoly1d(syy,szz,sorder,3,0.01,OGOODBAD=ogoodbad,/SILENT)
    
    for j = 0, stop-start do begin

        if finite(result[j,1]) ne 1 then goto, cont4
        if botedge[j]+ybuffer lt -0.5 then goto, cont4
        if topedge[j]-ybuffer gt nrows-0.5 then goto, cont4        

        y_bot = round( 0 > (botedge[j] +ybuffer) )
        y_top = round( (nrows-1) < (topedge[j] -ybuffer) )

        yy    = y[y_bot:y_top]

        if n_elements(yy) eq 1 then goto, cont4
        yy_arc = poly(yy,spixtoarc[j,*])
        spat = poly(yy_arc,coeff)

        flat[x[j],y_bot:y_top] = image[x[j],y_bot:y_top]/(spat*result[j,1])
        surf[x[j],y_bot:y_top] = (spat*result[j,1])

        cont4:

    endfor

    z = where(mask eq 1)
    robuststats,flat[z],3,mean,var,stddev,/SILENT
    print, 'RMS = ',stddev
    rms[i] = stddev

    if keyword_set(UPDATE) then begin
        
        if cancelbutton then begin
            
            cancel = progressBar->CheckCancel()
            if cancel then begin

                progressBar->Destroy
                obj_destroy, progressbar
                cancel = 1
                return, -1

            endif

        endif
        percent = (i+1)*(100./float(norders))
        progressbar->update, percent

    endif
    cont5:

endfor

if keyword_set(UPDATE) then begin

    progressbar-> destroy
    obj_destroy, progressbar
    
endif

z = where(flat le 0,count)
if count ne 0 then flat[z] = 1.

return, flat


end





