; NAME:
;     poly1d 
;
; PURPOSE:
;     To evaluate a polynomial function of one independent variable.
;
; CATEGORY:
;     Data Modeling
;
; CALLING SEQUENCE:
;     result = poly1d(x,coeffs,[coeff_var],CANCEL=cancel)
;
; INPUTS:
;     x         - Array of independent values
;     coeffs    - Coeffs of the polynomial
;     coeff_var - The variances of the coefficients
;
; OUTUTS:
;     Returns values of the polynomial evaluated at x
;
; KEYWORD PARAMETERS:    
;     CANCEL - Set on return if there is a problem
; 
; PRODEDURES CALLED:
;     Requires the Astronomy User's Library
;
; PROCEDURE:
;     Straight forward
;
;  MODIFICATION HISTORY:
;     2001-05-25 - Written by M. Cushing, Institute for Astronomy, UH
;
function poly1d,x,coeffs,coeffs_var,YVAR=yvar,CANCEL=cancel
;
cancel = 0

;  Check parameters

if n_params() lt 2 then begin
    
    cancel = 1
    print, 'Syntax -  result = poly1d(x,coeffs,coeffs_var,YVAR=yvar,$'
    print, '                          CANCEL=cancel)'
    return, -1

endif

zparcheck, 'poly1d', x, 1, [2,3,4,5], [0,1], 'Independent variable' 
zparcheck, 'poly1d', coeffs, 2, [2,3,4,5], [0,1], 'Coefficients' 

xx      = double(x)
ninvar  = n_elements(x)
ncoeffs = n_elements(coeffs)

y     = 0.
yvar  = 0.
for i = 0, ncoeffs-1 do begin

    y     = y + coeffs[i]*xx^i
    if n_elements(coeffs_var) ne 0 then yvar = yvar + coeffs_var[i]*(xx^(i))^2

endfor

return, y

end






