; NAME:
;     poly2d 
;
; PURPOSE:
;     To evaluate a polynomial function of two independent variables
;
; CATEGORY:
;     Data Modeling
;
; CALLING SEQUENCE:
;     result = poly2d(x,y,xorder,yorder,coeff,CANCEL=cancel)
;
; INPUTS:
;     x      - Array of first independent values
;     y      - Array of second independent values
;     xorder - Polynomial order in the x direction
;     yorder - Polynomial order in the y direction
;     coeff  - Coeffs of the polynomial
;
; OUTUTS:
;     Returns values of the polynomial evaluated at x,y
;
; KEYWORD PARAMETERS:    
;     CANCEL - Set on return if there is a problem
; 
; PRODEDURES CALLED:
;     Requires the Astronomy User's Library
;
; PROCEDURE:
;     For small data sets, the design matrix A could be constructed
;     and the results determined by A ## coeff = b.  For large data
;     sets, such as surface fits to large images, this is
;     unreasonable since the design matrix would be VERY large.
;     The only other way is to loop over the individual
;     data  points. -> A[*,0] * coeff[0] = b[0]. 
;
;  MODIFICATION HISTORY:
;     2000-06-18 - Written by M. Cushing, Institute for Astronomy, UH
;
function poly2d,x,y,xorder,yorder,coeff,CANCEL=cancel
;
cancel = 0

;  Check parameters

if n_params() lt 5 then begin
    
    cancel = 1
    print, 'Syntax -  poly2d,x,y,xorder,yorder,coeff,CANCEL=cancel'
    return, -1

endif

zparcheck, 'poly2d', x, 1, [2,3,4,5], [0,1], 'First independent variables' 
zparcheck, 'poly2d', y, 2, [2,3,4,5], [0,1], 'Second independent variables' 
zparcheck, 'poly2d', xorder, 4, [2,3,4,5], 0, 'X fit order' 
zparcheck, 'poly2d', yorder, 5, [2,3,4,5], 0, 'Y fit order'
zparcheck, 'poly2d', coeff, 6, [2,3,4,5], 1, 'Polynomial coefficients' 

xx      = double(x)
yy      = double(y)
ninvar  = n_elements(x)
ncoeffs = (xorder+1)*(yorder+1)

xexp  = findgen(xorder+1)
yexp  = findgen(yorder+1)
z     = dblarr(ninvar)    
    
for i = 0L, long(ninvar-1) do z[i] = total((xx[i]^xexp # yy[i]^yexp ) * coeff)

return, z

end






