; NAME:
;     rdll (Read Line List)
;
; PURPOSE:
;     To read a line list into an array.
;       
; CATEGORY:
;     Data Reduction  
;
; CALLING SEQUENCE:
;     lines = rdll(filename,RANGE=range,RES=RES,THRESH=thresh,VACUUM=vaccum)
;
; INPUTS:
;     filename - A string containing the full path of a file
;                containing a line list with air wavelengths in
;                units of Angstroms, e.g. /scr1/junk/argon_master.
;
; OUTUTS:
;     Return an array of the lines (in microns) which conform to the keywords.
;     The default is to return the wavelengths in air.
;
; KEYWORD PARAMETERS:    
;     RES    - If the resolution is given, the list is pruned to avoid
;              lines that are too close together.
;     RANGE  - Range of wavelengths in microns to return.
;     VACCUM - Set to return the wavelengths in vaccum.
;     CANCEL - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;
; PROCEDURE:
;     The lines are read in, converted to microns, and then pruned 
;     based on the resolution
;
; REVISION HISTORY:
;     2000-08-25 - Written by M. Cushing, Institute for Astronomy, UH
;
function rdll,filename,RANGE=range,RES=res,THRESH=thresh,VACUUM=vacuum,$
              CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 1 then begin
    
    print, 'Syntax - result = rdll(filename,RANGE=range,RES=res,$'
    print, '                       THRESH=thresh,VACUUM=vacuum,CANCEL=cancel)'
    cancel = 1
    return, -1

endif
zparcheck, 'rdll',filename, 1,7, 0, 'filename'

;  Read entire file.

rdfloat,filename,lines,/DOUBLE

if keyword_set(VACUUM) then begin

    airtovac,lines

endif

lines = temporary(lines)/10000.D

if n_elements(RANGE) ne 0 then begin

    z = where(lines ge range[0] and lines le range[1])
    lines = lines[z]

endif

;  Prune the line list based on the resolution R.

if n_elements(RES) ne 0 then begin

    if n_elements(THRESH) eq 0 then thresh = 1.

    nlines  = n_elements(lines)
    goodbad = intarr(nlines)+1

    for i = 1,nlines-2 do begin

        delta = lines[i]/float(RES)
        z = where(lines ge lines[i]-thresh*delta/2.534 and $
                  lines le lines[i]+thresh*delta/2.534,count)
        if count gt 1 then begin

;            print, lines[i]
;            print, lines[z]
;            re= ' '
;            read, re
            goodbad[z] = 0
            
        endif
    endfor
    good = where(goodbad eq 1)
    lines = lines[good]

endif else goodbad = replicate(1,n_elements(lines))

return, lines

end


