; NAME:
;     readcal
;
; PURPOSE:
;     To read a calibration file for Spextool.
;       
; CATEGORY:
;     Data Reduction
;
; CALLING SEQUENCE:
;     readcal,filename,modename,rotation,norders,orders,start,stop,slith_pix,$
;             slith_arc,edgedeg,sg_width,sg_deg,spatdeg,fixed,guesspos,$
;             CANCEL=cancel
;
; INPUTS:
;     filename  - The name of a Spextool calibration file.
;
; OUTUTS:
;     modename  - A string giving the mode name.
;     rotation  - The IDL rotation direction (See rotate in IDL help page)
;     norders   - The number of orders in the image.
;     orders    - An array of the order numbers.
;     start     - The starting column.
;     stop      - The ending column.
;     slith_pix - The slit height in pixels. 
;     slith_arc - The slit height in arcseconds. 
;     edgedeg   - The polynomial degree of the edges of the orders.
;     sg_width  - The width in pixels of the smoothing window for the 
;                 Savitzky-Golay smoothing routine.
;     sg_deg    - The polynomial degree of the Savitzky-Golay
;                 smoothing rountine.
;     spatdeg   - The polynomial degree in the spatial dimension.
;     fixed     - If 'Yes' then the guesspos are row numbers of the 
;                 edge of the order.  If 'No', then they are guess positions.
;     guesspos  - An (2,norders) array of positions of the orders on
;                 the array.
;
; KEYWORD PARAMETERS:    
;     CANCEL - Set on return if something is wrong.
;
; PROCEDURE'S USED:
;     Requires the Astronomy User's Library
;
; PROCEDURE:
;     Obvious
;
; REVISION HISTORY:
;     2000-05-10 - Written by M. Cushing, Institute for Astronomy, UH
;
pro readcal,filename,modename,rotation,norders,orders,start,stop,slith_pix,$
            slith_arc,edgedeg,sg_width,sg_deg,spatdeg,fixed,guesspos,$
            CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 1 then begin
    
    cancel = 1
    print, 'Syntax - readcal,filename,modename,rotation,norders,orders,$'
    print, '                 start,stop,slith_pix,slith_arc,edgedeg,$'
    print, '                 sg_width,sg_deg,spatdeg,guesspos,CANCEL=cancel'
    return

endif

zparcheck, 'readcal', filename, 1,7,0, 'Filename' 

line = ''
openr, lun, filename,/GET_LUN

readf, lun, line
modename = strtrim( (strsplit(line,'=',/EXTRACT))[1],2 )

readf, lun, line
rotation = fix( (strsplit(line,'=',/EXTRACT))[1] )

readf, lun, line
orders = ( strsplit(line,'=',/EXTRACT) )[1]
orders = fsextract(orders,/INDEX,CANCEL=cancel)
if cancel then return
norders = n_elements(orders)

readf, lun, line
start = fix( ( strsplit(line,'=',/EXTRACT) )[1] )

readf, lun, line
stop = fix( ( strsplit(line,'=',/EXTRACT) )[1] )

readf, lun, line
slith_pix = fix( ( strsplit(line,'=',/EXTRACT) )[1] )

readf, lun, line
slith_arc = float( ( strsplit(line,'=',/EXTRACT) )[1] )

readf, lun, line
edgedeg = fix( ( strsplit(line,'=',/EXTRACT) )[1] )

readf, lun, line
sg_width = fix( ( strsplit(line,'=',/EXTRACT) )[1] )

readf, lun, line
sg_deg = fix( ( strsplit(line,'=',/EXTRACT) )[1] )

readf, lun, line
spatdeg = fix( ( strsplit(line,'=',/EXTRACT) )[1] )
 
readf, lun, line
fixed = strtrim( ( strsplit(line,'=',/EXTRACT) )[1],2)

guesspos = intarr(2,norders)

for i = 0, norders-1 do begin

    readf, lun, line
    tmp = ( strsplit(line,'=',/EXTRACT) )[1] 
    guesspos[*,i] = fix( strsplit(tmp,' ',/EXTRACT) )   
    
endfor


free_lun, lun




end
