; NAME:
;     readmfits
;
; PURPOSE:
;     To read multiple FITS images into memory.
;
; CATEGORY:
;     Data Input
;
; CALLING SEQUENCE:
;     readmfits,files,data,[hdrinfo],[var],KEYWORDS=keywords,ROTATE=rotate,$
;               PAIR=PAIR,ITIME=itime,COADDS=coadds,NDRS=ndrs,GAIN=gain,$
;               READNOISE=readnoise,WIDGET_ID=widget_id,NIMAGES=nimages,$
;               CANCEL=cancel
;
; INPUTS:
;     files - A string of (fullpath) file names.
; 
; OUTUTS:
;     Returns a floating array of images
;
; OPTIONAL OUTUTS:
;     hdrinfo - If KEYWORDS are given, an array of structures of the 
;               requested hdr keywords for each image.  If no
;               keywords are given, then all the keywords are
;               returned in a structure.
;     var     - A floating array of variance images is returned
;
; KEYWORD PARAMETERS:   
;     KEYWORDS  - A string array of keywords to extract from the hdrs. 
;     ROTATE    - Set to the desired IDL rotation command (ROTATE).
;     PAIR      - Set to pair subtract.  Must be even number of input files.
;     ITIME     - The itime keyword
;     COADDS    - The coadds keyword
;     NDRS      - The ndrs keyword
;     GAIN      - The gain in electrons per DN
;     READNOISE - The rms readnoise in electrons
;     WIDGET_ID - If given, an pop-up error message will appear over
;                 the widget.
;     NIMAGES   - The number of images in the output data
;     CANCEL    - Will be set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     gethdrinfo
;
;  PROCEDURE: 
;     Images are read in, pair subtracted if requested, and
;     variances images are computed if requested.
;
;REVISION HISTORY:
;     2000-08-21 - Written by M. Cushing, Institute for Astronomy, UH
;     2000-11-17 - Fixed bug that screwed up multiple pairs reading.
;     2000-11-18 - Added NIMAGES keyword.
;     2001-01-20 - Modified the way the hdr structures are
;                  constructed.  Appears to have speed things up.
;                - Changed the keywords input to be a keyword.
;     2001-01-30 - Modified to create variance images and added 
;                  the COADDS, NDRS, ITIME, and GAIN keywords.
;     2001-02-25 - Consolidated the COADDS, NDRS, and ITIME keywords
;                  into the DIV keyword.
;     2001-04-20 - Modified the header storing to speed things up, 
;                  now uses copy_struct_inx.
;     2001-05-15 - Added the READNOISE.
;       
pro readmfits,files,data,hdrinfo,var,KEYWORDS=keywords,ROTATE=rotate,$
              PAIR=PAIR,ITIME=itime,COADDS=coadds,NDRS=ndrs,GAIN=gain,$
              READNOISE=readnoise,WIDGET_ID=widget_id,NIMAGES=nimages,$
              CANCEL=cancel

cancel  = 0

;  Check parameters

if n_params() lt 1 then begin
    
    cancel = 1
    print, 'Syntax - readmfits,files,data,hdrinfo,var,KEYWORDS=keywords,$'
    print, '                   ROTATE=rotate,PAIR=PAIR,ITIME=itime,$'
    print, '                   COADDS=coadds,NDRS=ndrs,GAIN=gain,$'
    print, '                   READNOISE=readnoise,WIDGET_ID=widget_id,$'
    print, '                   NIMAGES=nimages,CANCEL=cancel'
    return

endif

zparcheck, 'readmfits',files, 1,7,[0,1], 'File Names' 

;  Get setup info.

error     = (arg_present(var) eq 1) ? 1:0
readnoise = (n_elements(READNOISE) ne 0) ? float(readnoise):0.
rot       = (n_elements(ROTATE) ne 0) ? rotate:0
gain      = (n_elements(GAIN) ne 0) ? float(gain):1.

;  Get number of images and check to make sure even number if /PAIR.

nfiles = n_elements(files)
if keyword_set(PAIR) then begin

    result = crange(nfiles,0,'Number of Files',/EVEN,WIDGET_ID=widget_id,$
                    CANCEL=cancel)
    if cancel then return
    nimages = nfiles/2

endif else nimages = nfiles

;  Get size of images, itime, coadds, and ndrs from header.

testhdr = headfits(files[0])
NAXIS1  = fxpar(testhdr,'NAXIS1')
NAXIS2  = fxpar(testhdr,'NAXIS2')
ITIME   = (n_elements(ITIME) ne 0) ? fxpar(testhdr,ITIME):1.
COADDS  = (n_elements(COADDS) ne 0) ? fxpar(testhdr,COADDS):1.
NDRS    = (n_elements(NDRS) ne 0) ? fxpar(testhdr,NDRS):1.

COADDS  = (COADDS eq 0) ? 1:COADDS
NDRS    = (NDRS eq 0) ? 1:NDRS

divisor = ITIME*COADDS*NDRS
rdvar   = (2.*readnoise^2)/NDRS/COADDS/ITIME^2/gain^2

;  Make data arrays.

data              = fltarr(NAXIS1,NAXIS2,nimages)
hdrinfo           = replicate(gethdrinfo(testhdr,keywords),nfiles)
if error then var = fltarr(NAXIS1,NAXIS2,nimages)

;  Read images

if keyword_set(PAIR) then begin

    for i = 0, nimages-1 do begin
        
        Aimage = rotate(readfits(files[i*2],A_hdr),rot) > 0
        Bimage = rotate(readfits(files[i*2+1],B_hdr),rot) > 0
        
        data[*,*,i]              = (Aimage-Bimage)/divisor
        if error then var[*,*,i] = (Aimage+Bimage)*NDRS/gain/divisor^2+$
          2.*rdvar

        copy_struct_inx,gethdrinfo(A_hdr,keywords),hdrinfo,index_to=i*2
        copy_struct_inx,gethdrinfo(B_hdr,keywords),hdrinfo,index_to=i*2+1
        
    endfor

endif

if not keyword_set(PAIR) then begin

    for i = 0, nimages-1 do begin

        img = rotate(readfits(files[i],hdr),rot) > 0
        data[*,*,i] = float(img)/divisor
        if error then var[*,*,i] = (img*NDRS/gain/divisor^2)+rdvar

        copy_struct_inx,gethdrinfo(hdr,keywords),hdrinfo,index_to=i

    endfor

endif

end






