; NAME:
;     readspec
;
; PURPOSE:
;     To read a SpeX spectral FITS image.
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     readspec,specname,spec,hdr,obsmode,start,stop,norders,naps,orders,$
;              xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
;              airmass,CANCEL=cancel
; INPUTS:
;     flatname - A string name of the spectrum
;
; OUTUTS:
;     spec      - The spectrum [stop-start+1,3,norders*naps]
;     hdr       - A string array of the FITS header
;     obsmode   - The obsmode
;     start     - Start column
;     stop      - Stop column
;     norders   - Number of orders
;     naps      - Number of apertures
;     orders    - An array [norders] of the order numbers
;     xunits    - A string giving the units of the x-axis
;     yunits    - A string giving the units of the y-axis
;     slith_pix - The approximate slit height in pixels
;     slith_arc - The slit height in arcseconds
;     slitw_pix - The slit width in pixels
;     slitw_arc - The slit width in arcseconds
;     airmass   - The average airmass of the observation  
;
; KEYWORD PARAMETERS:    
;     CANCEL - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires IDL 5.3 or greater due to changes in string procedures.
;     Requires the Astronomy User's Library.
;
; PROCEDURE:
;     Straight forward
;
;REVISION HISTORY:
;     2000 - Written by M. Cushing, Institute for Astronomy, UH
;
pro readspec,specname,spc,hdr,obsmode,start,stop,norders,naps,orders,$
             xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
             airmass,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 1 then begin

    print, 'Syntax - readspec,specname,spc,hdr,obsmode,start,stop,norders,$'
    print, '                  naps,orders,xunits,yunits,slith_pix,slith_arc,$'
    print, '                  slitw_pix,slitw_arc,airmass,CANCEL=cancel'
    cancel = 1
    return

endif
zparcheck, 'readspec', specname, 1, 7, 0, 'Specname'

spc = readfits(specname,hdr)

;  Get hdr info.

xunits = strcompress(fxpar(hdr,'XUNITS'),/re)
yunits = strcompress(fxpar(hdr,'YUNITS'),/re)

norders  = fxpar(hdr,'NORDERS')
naps     = fxpar(hdr,'NAPS')
start    = fxpar(hdr,'START')
stop     = fxpar(hdr,'STOP')

obsmode  = strcompress( fxpar(hdr, 'MODENAME'), /re )
orders   = long( strsplit( fxpar(hdr,'ORDERS'), ',', /extract) )

slith_pix = fxpar(hdr,'SLTH_PIX')
slith_arc = fxpar(hdr,'SLTH_ARC')
slitw_pix = fxpar(hdr,'SLTW_PIX')
slitw_arc = fxpar(hdr,'SLTW_ARC')

airmass   = fxpar(hdr,'AIRMASS')


end
