; NAME:
;     robustsfactor
;
; PURPOSE:
;     To robustly determine the scale factor between two spectra.
;
; CATEGORY:
;     Data Modeling
;
; CALLING SEQUENCE:
;     scale = robustsfactor(refspec,spec,thresh,eps,WARNING=warning,$
;                           LIMIT=limit,CANCEL=cancel)
;
; INPUTS:
;     refspec - The reference spectrum.
;     spec    - The object spectrum.
;     thresh  - Sigma threshold to identify outliers.
;     eps     - Limit of the fractional change in the standard
;               deviation of the fit.
; 
; OUTUTS:
;     Returns the scale factor.
;
; KEYWORD PARAMETERS:    
;     WARNING - Set on return if the scale factor is gt limit or lt 1/limit.
;     LIMIT   - Limit on the scale factor.  If the scale factor is 
;               gt limit or lt 1/limit, a warning message is printed.
;     CANCEL  - Set on return if something is wrong.
;
;  PROCEDURES CALLED:
;     Requires the Astronomy User's Library.
;
;  PROCEDURE: 
;     Later
;
;  MODIFICATION HISTORY:
;     Written 2001-07-03 by M. Cushing, Institute for Astronomy, UH
;             2001-08-18 - Added WARNING and LIMIT keyword.
;
function robustsfactor,refspec,spec,thresh,eps,WARNING=warning,LIMIT=limit,$
                       CANCEL=cancel

cancel = 0
ittr   = 0

;  Check parameters

if n_params() lt 2 then begin
    
    print, 'Syntax - result = robustsfactor(spec1,spec2,CANCEL=cancel)'
    return, cancel

endif
zparcheck, 'robustsfactor', refspec, 1, [2,3,4,5], 1, 'Reference Spectra' 
zparcheck, 'robustsfactor', spec, 2, [2,3,4,5], 1, 'Obj Spectra' 

WARNING = 0

;  Get rid of NaNs

z = where(finite(refspec) eq 1 and finite(spec) eq 1)
refspec2 = refspec[z]
spec2    = spec[z]

refspec2 = median(refspec2,5)
spec2    = median(spec2,5)

;  Do a first pass fit to the data.

count_initgood = n_elements(spec2)

scale = total(spec2*refspec2)/total(spec2^2)
if scale eq 1.0 then goto, cont1

test = refspec2 - scale*spec2
mask = findoutliers(test,3,CANCEL=cancel)
z = where(mask eq 1,count)
moments, test[z],mean,dummy,sig

good = where(abs( (test-mean)/sig ) le thresh,count_good)

;  If it doesn't find any outliers then go back, if not continue.

if count_good eq count_initgood then goto, cont1

repeat begin

    ittr     = ittr + 1
    sig_old  = sig

    scale = total(spec2[good]*refspec2[good])/total((spec2[good])^2)
    if scale eq 1.0 then goto, cont1
    test = refspec2[good]-double(scale)*double(spec2[good])
    mask = findoutliers(test,3,CANCEL=cancel)
    z = where(mask eq 1,count)
    moments, test[z],mean,dummy,sig,/DOUBLE
    if ((sig_old-sig)/sig_old) lt eps then goto, cont1
    test = refspec2 - scale*spec2
    good = where(abs( (test-mean)/sig ) le thresh,count_good)

endrep until ittr eq 10

cont1:

if n_elements(LIMIT) ne 0 then begin

    if scale gt limit or scale lt 1./float(limit) then begin
        
        print, 'Warning:  Scale factor ='+strcompress(scale,/RE)
        WARNING = 1
        
    endif

endif

return, scale

end

