; NAME:
;     wavecal
;
; PURPOSE:
;     To wavelength calibrate either single order or X-dispersed spectra.
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     result = wavecal(x,spectra,norders,naps,ltop,pixoffset,orders,$
;                      homeorder,FWHM,disdeg,orderdeg,lines,robustthresh,$
;                      RMS=rms,UPDATE=update,WIDGET_ID=widget_id,PLOT=plot,$
;                      PS=ps,CANCEL=cancel)
;
; INPUTS :
;     x            - An array of independent values for the spectra
;     spectra      - An array [nelements,naps*norders] where array[*,x]
;                    are flux values
;     norders      - The number of orders
;     naps         - The number of apertures
;     ltop         - (lambda to pixel) 1-D polynomial guess
;                    to convert wavelengths to pixel positions
;     orders       - An array of the orders numbers
;     home         - The home order number (see below).
;     FWHM         - The FWHM of the gaussian to fit to each line. 
;     pixdegree    - The order to fit the results in the pixel
;                    dimension.
;     orderdegree  - The order to fit the results in the order
;                    dimension.  If a single order, set to 0.
;     lines        - An array of lines that appear on the array
;     robustthresh - The sigma threshold to identify outliers in the
;                    robust polynomial fit.
;
; OUTUTS:
;     Returns the 2-D coefficients of the superfit.
;
; KEYWORD PARAMETERS:    
;     RMS       - Returns the rms of the robust fit(s).
;     UPDATE    - If set, the program will launch the Fanning
;                 showprogress widget.
;     WIDGET_ID - If given, a cancel button is added to the Fanning
;                 showprogress routine.  The widget blocks the
;                 WIDGET_ID, and checks for for a user cancel
;                 command.
;     PLOT      - Set to plot the results.
;     PS        - Set to plot the results to a postscript file
;     CANCEL    - Set on return if there is a problem.
;
; PROCEDURE:
;     This routine is based on the idea that all the orders of 
;     an cross-dispersed spectrum can be wavelength calibrated at
;     once.
;
; REVISION HISTORY:
;     2000-08-01 - Written by M. Cushing, Institute for Astronomy, UH
;     2001-02-20 - Heavily modified to make sure that only lines found 
;                  in all apertures are used in the fit.
;     2001-03-05 - Modified to identify the lines in a given order
;                  without knowing the wavelength range of the order.
;
function wavecal,x,spectra,norders,naps,ltop,pixoffset,orders,$
                 homeorder,FWHM,disdeg,orderdeg,lines,robustthresh,RMS=rms,$
                 UPDATE=update,WIDGET_ID=widget_id,PLOT=plot,PS=ps,$
                 CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 13 then begin
    
    print, 'Snytax - result = wavecal(x,spectra,norders,naps,ltop,pixoffset,$'
    print, '                          orders,homeorder,FWHM,disdeg,$'
    print, '                          orderdeg,lines,robustthresh,RMS=rms,$'
    print, '                          UPDATE=update,WIDGET_ID=widget_id,$'
    print, '                          PLOT=plot,PS=ps,CANCEL=cancel)'
    cancel = 1
   return, 1

endif

zparcheck, 'wavecal', x, 1, [2,3,4,5], 1, 'X'
zparcheck, 'wavecal', spectra, 2, [2,3,4,5], [1,2], 'Spectra'
zparcheck, 'wavecal', norders, 3, [2,3,4,5], 0, 'Norders'
zparcheck, 'wavecal', naps, 4, [2,3,4,5], 0, 'Naps'
zparcheck, 'wavecal', ltop, 5, [2,3,4,5], [1,2], 'Ltop'
zparcheck, 'wavecal', pixoffset, 6, [2,3,4,5], 0, 'Offset'
zparcheck, 'wavecal', orders, 7, [2,3,4,5], 1, 'Orders'
zparcheck, 'wavecal', homeorder, 8, [2,3,4,5], 0, 'home'
zparcheck, 'wavecal', FWHM, 9, [2,3,4,5], 0, 'FWHM'
zparcheck, 'wavecal', disdeg, 10, [2,3,4,5], 0, 'Dispersion degree'
zparcheck, 'wavecal', orderdeg, 11, [2,3,4,5], 0, 'Order degree'
zparcheck, 'wavecal', lines, 12, [2,3,4,5], 1, 'Lines'
zparcheck, 'wavecal', robustthresh, 13, [2,3,4,5], 0, 'Robustthresh'

;  Initialize arrays.

rmss     = dblarr(naps)

suparray = ptrarr(naps,/ALLOCATE_HEAP)

if keyword_set(UPDATE) then begin
    
    Message = 'Identifying Lines...'
    cancelbutton = (n_elements(WIDGET_ID) ne 0) ? 1:0
    progressbar = obj_new('SHOWPROGRESS',widget_id,color=2,$
                          CANCELBUTTON=cancelbutton,$
                          MESSAGE=message)
    progressbar -> start
    
endif

;  Start calibration.

for i = 0, norders-1 do begin

;  Identify the lines in orders[i] and scale them to the home order.

    xguess = poly(lines,ltop[*,i]) + pixoffset        
    z      = where(finite(spectra[*,i*naps]) eq 1,count)
    good   = where(xguess gt x[z[0]] and xguess lt x[z[count-1]])
    xguess = xguess[good]
    rlines = lines[good]
    slines = rlines / ( float(homeorder)/float(orders[i]) ) 

    sxpos    = fltarr(naps,n_elements(xguess))
    sgoodbad = intarr(naps,n_elements(xguess))

    for j = 0, naps-1 do begin
        
        spec = reform( spectra[*,i*naps+j] )
        good = findlines(x,spec,xguess,FWHM/2.354,GOODBAD=goodbad,$
                         PLOT=0,WLINES=rlines)
        
        sxpos[j,*]    = good
        sgoodbad[j,*] = goodbad
        
    endfor
    if naps ne 1 then good = total(sgoodbad,1) else good = sgoodbad
    z    = where(good eq naps,count)

    for j = 0, naps-1 do begin

        array = transpose( [[reform(sxpos[j,z])],[replicate(orders[i],count)],$
                 [reform(slines[z])]] )
        *suparray[j] = (i eq 0) ? array:[[*suparray[j]],[array]]

    endfor

    if keyword_set(UPDATE) then begin
            
        if cancelbutton then begin
            
            cancel = progressBar->CheckCancel()
            if cancel then begin
                
                progressBar->Destroy
                obj_destroy, progressbar
                cancel = 1
                return, -1
                
            endif
            
        endif
        percent = (i+1)*(100./float(norders))
        progressbar->update, percent
        
    endif
    
endfor

if keyword_set(UPDATE) then begin
    
    progressbar-> destroy
    obj_destroy, progressbar
    
endif


bad:

scoeffs = dblarr( (disdeg+1)*(orderdeg+1),naps )
for i = 0, naps-1 do begin

    supxpos  = reform( (*suparray[i])[0,*] )
    suporder = reform( (*suparray[i])[1,*] )
    suplines = reform( (*suparray[i])[2,*] )

;    file = 'data_ap'+string(i+1,format='(i2.2)')+'.dat'
;    openw, 10, file
;    for k = 0, n_elements(suporder)-1 do printf, 10, supxpos[k],suporder[k],$
;      suplines[k]
;    close, 10
    
    supxpos  = supxpos[1:*]
    suplines = suplines[1:*]
    suporder = suporder[1:*]
    z        = sort(suplines)
    supxpos  = supxpos[z]
    suplines = suplines[z]
    suporder = suporder[z]

    coeffs = robustpoly2d(supxpos,suporder,suplines,disdeg,orderdeg,$
                          robustthresh,0.1,SILENT=0,RMS=rms,$
                          SIGMA=coeff_sigma,OGOODBAD=ogoodbad)
    z = where(finite(coeff_sigma) eq 0,count)
    if count ne 0 then begin

        orderdeg = 0 > (orderdeg-1)
        goto, bad

    endif
    scoeffs[*,i] = coeffs
    rmss[i] = rms

    if keyword_set(PLOT) or keyword_set(PS) then begin
        
        if keyword_set(PLOT) then begin
            
            result = get_screen_size()
;            window, /free, ysize=result[1]-50
            window, /free
            win_idx = !d.window

        endif else begin
            
            set_plot, 'ps'
            filename = 'wavecal_ap'+string(i+1,format='(i2.2')+'.ps'
            device, filename=filename,/inches,ysize=10,yoffset=0.5,/color
    
        endelse
;        !p.multi[1] = 1
;        !p.multi[2] = 2
;        !p.multi[0] = 2
        
;  Plot the top window of x versus wavelength.
        
        plines = poly2d(supxpos,suporder,disdeg,orderdeg,coeffs)
    
;        title = '!5Wavelength Calibration - Aperture '+strtrim(i+1,2)
;        plot,supxpos,suplines,/xsty,/ysty,xtitle='!5Pixels',$
;          ytitle='Wavelength (microns)',psym=1,title=title
;        oplot,supxpos,plines,color=2
        
;        z = where(ogoodbad eq 0,count)
;        if count ne 0 then oplot,supxpos[z],suplines[z],color=4,psym=1
        
;  Now plot the residuals in the second window.
        
        del = (plines - suplines)*1e4
        rmsdev = rms[n_elements(rms)-1]*10000.
        title = 'RMS = '+string(rmsdev,format='(f6.3)')+'A, '+$
          strcompress(fix(disdeg),/re)+'x'+strcompress(fix(orderdeg),/re)+$
          ' fit'
        
        plot,supxpos,del,/xsty,/ysty,xtitle='!5Pixels',$
          ytitle='Deviation (!6!sA!r!u!9 %!6!n)',psym=1,/nodata,title=title,$
          yrange=[-4.*rms[n_elements(rms)-1]*1e4,$
                  4.*rms[n_elements(rms)-1]*1e4]
        
        for k = 0, norders-1 do begin
            
            z = where(suporder eq orders[k],count)
            if count ne 0 then oplot,supxpos[z],del[z],psym=1,color=k+2
            
        endfor
        
        z = where(ogoodbad eq 0,count)
        if count ne 0 and keyword_set(PS) then oplot,supxpos[z],del[z],$
          psym=4,symsize=1.5,color=0
        if count ne 0 and keyword_set(PLOT) then oplot,supxpos[z],del[z],$
          psym=4,symsize=1.5,color=1
        
        if keyword_set(PLOT) then plots,!x.crange,[0,0],color=1
        if keyword_set(PS)   then plots,!x.crange,[0,0],color=0
        
        for k = 0, norders-1 do begin
            
            name = (k eq 0) ? 'Order '+strcompress(orders[k],/re):$
              [name,'Order '+strcompress(orders[k],/re)]
            color = (k eq 0) ? (k+2):[color,(k+2)]
            
        endfor
        legend,name,psym=intarr(norders)+1,color=color,/right,/top
        
        if keyword_set(PS) then begin
            
            device, /close
            set_plot, 'x'
            
        endif 
        !p.multi=0
        
    endif

endfor

rms    = rmss
coeffs = scoeffs[*,naps-1]

return,scoeffs

ptr_free, suparray
suparray = 1B

end

