; NAME:
;     writeflat
;
; PURPOSE:
;     To write a fits image of a normalized spectral flat-field
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     writeflat,normflat,hdrinfo,rotation,orders,edgecoeffs,xranges,slith_pix,$
;               slith_arc,slitw_pix,slitw_arc,modename,rms,outname,$
;               HISTORY=history,CANCEL=cancel
;
; INPUTS:
;     normflat   - A 2-D normalized flat field
;     hdrinfo    - A structure with FITS keywords and values to be
;                  written to the header
;     rotation   - The IDL rotation value
;     orders     - An array of order numbers 
;     edgecoeffs - Array [degree+1,2,norders] of polynomial coefficients 
;                  which define the edges of the orders.  array[*,0,0]
;                  are the coefficients of the bottom edge of the
;                  first order and array[*,1,0] are the coefficients 
;                  of the top edge of the first order.
;     xranges    - An array [2,norders] of pixel positions where the
;                  orders are completely on the array
;     slith_pix  - The slit length in pixels
;     slith_arc  - The slit length in arcseconds
;     slitw_pix  - The slit width in pixels
;     slitw_arc  - The slit width in arcseconds
;     modename   - The name of the observing mode
;     rms        - An array of the rms values of the flat-field
;     outname    - String of the name of the fits image
;
;
; OUTUTS:
;     Writs a FITS file to disk.
;
; KEYWORD PARAMETERS:    
;     HISTORY - A string describing the processes of the flats
;     CANCEL  - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library.
;
; PROCEDURE:
;     Straight forward
;
; MODIFICATION HISTORY:
;     2000-08-25 - Written by M. Cushing, Institute for Astronomy, UH
;     2001-10-04 - Added the xranges inputs and removed the start and stop
;     2001-11-05 - Added the rotation input
;
pro writeflat,normflat,hdrinfo,rotation,orders,edgecoeffs,xranges,slith_pix,$
              slith_arc,slitw_pix,slitw_arc,modename,rms,outname,$
              HISTORY=history,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 13 then begin
    
    print, 'Syntax - writeflat,normflat,hdrinfo,rotation,orders,edgecoeffs,$'
    print, '                   xranges,slith_pix,slith_arc,slitw_pix,$'
    print, '                   slitw_arc,modename,rms,outname,$'
    print, '                   HISTORY=history,CANCEL=cancel'

    cancel = 1
    return

endif
zparcheck, 'writeflat', normflat, 1, [2,3,4,5], 2, 'Normflat'
zparcheck, 'writeflat', hdrinfo, 2, 8, [0,1], 'Hdrinfo'
zparcheck, 'writeflat', rotation, 3, [2,3,4,5], 0, 'Rotation'
zparcheck, 'writeflat', orders, 4, [2,3,4,5], 1, 'Orders'
zparcheck, 'writeflat', edgecoeffs, 5, [2,3,4,5], [2,3], 'Normflat
zparcheck, 'writeflat', xranges, 6, [2,3,4,5],[1,2], 'Xranges'
zparcheck, 'writeflat', slith_pix, 7, [2,3,4,5], 0, 'Slith_pix'
zparcheck, 'writeflat', slith_arc, 8, [2,3,4,5], 0, 'Slith_arc'
zparcheck, 'writeflat', slitw_pix, 9, [2,3,4,5], 0, 'Slitw_pix'
zparcheck, 'writeflat', slitw_arc, 10, [2,3,4,5], 0, 'Slitw_arc'
zparcheck, 'writeflat', modename, 11, 7, 0,'Modename'
zparcheck, 'writeflat', rms, 12, [2,3,4,5], 1, 'RMS'
zparcheck, 'writeflat', outname, 13, 7,0, 'Outname'


;  Get norders, the fitorder and the number of tags in the hdrinfo struc.

norders  = n_elements(orders)
fitorder = (size(edgecoeffs))[1]
ntags    = n_tags(hdrinfo)

;  Make hdr with hdrinfo in it.

fxhmake,hdr,normflat

names = tag_names(hdrinfo)
for i = 0, ntags - 1 do begin

    fxaddpar,hdr,names[i],hdrinfo.(i)

endfor

fxaddpar,hdr,'MODENAME',modename
fxaddpar,hdr,'EDGEDEG',fitorder-1
fxaddpar,hdr,'NORDERS',norders
fxaddpar,hdr,'ORDERS',strjoin(strcompress(orders,/re),',')
fxaddpar,hdr,'SLTH_PIX',slith_pix
fxaddpar,hdr,'SLTH_ARC',slith_arc
fxaddpar,hdr,'SLTW_PIX',slitw_pix
fxaddpar,hdr,'SLTW_ARC',slitw_arc
fxaddpar,hdr,'ROTATION',rotation

;  Now add rms's.

for i = 0,norders-1 do begin

    name = 'ODR'+string(orders[i],format='(i2.2)')+'RMS'
    fxaddpar,hdr,name,rms[i]

endfor

;  Now add the xranges

for i = 0, norders-1 do begin

    name = 'ODR'+string(orders[i],format='(i2.2)')+'_XR'
    sxaddpar,hdr,name,strjoin(strtrim(xranges[*,i],2),',',/SINGLE)

endfor

;  Now add edge coeff

index = intarr(norders)
for i = 0,norders-1 do begin

    for j = 0, 1 do begin

        for k = 0, fitorder-1 do begin

            if j eq 0 then name = 'ODR'+string(orders[i],format='(i2.2)') + $
                                  '_B'+string(k+1,format='(i1.1)')

            if j eq 1 then name = 'ODR'+string(orders[i],format='(i2.2)') + $
                                  '_T'+string(k+1,format='(i1.1)')

            sxaddpar,hdr,name,edgecoeffs[k,j,i]

        endfor

    endfor


endfor

if n_elements(HISTORY) ne 0 then begin

    length = strlen(history)
    loop = ceil(float(length)/65.)
    for i = 0, loop-1 do begin
        
        hist = strmid(history,65*i,65)
        fxaddpar,hdr,'HISTORY',hist
        
    endfor
    
endif

writefits,outname,normflat,hdr

end

