; NAME:
;     writespec_xs
;
; PURPOSE:
;     To write extended source spectra to disk
;       
; CATEGORY:
;     data reduction
;
; CALLING SEQUENCE:
;     writespec_xs,spectra,xunits,yunits,fullpath,aimage,sky,flat,naps,orders,$
;                  start,stop,hdrinfo,apradius,mode,slith_pix,slith_arc,$
;                  slitw_pix,slitw_arc,version,BGR=bgr,BGORDER=bgorder,$
;                  TRACEFILE=tracefile,ARCIMAGE=arcimage,DISP=disp,FITS=fits,$
;                  TEXT=text,RMS=rms,RES=res,CANCEL=cancel
; INPUTS:
;     spectra   - An array [*,3,naps*norders] array of spectra where
;                 array [*,0,0] = wavelengths
;                 array [*,1,0] = flux
;                 array [*,2,0] = error
;     xunits    - A string giving the units of array[*,0,0]
;     yunits    - A string giving the units of array[*,1,0]
;     fullpath  - The fullpath of the file to be written to
;     aimage    - A string of the name if the Aimage
;     sky       - A string of the sky image
;     flat      - A string of the flat field image
;     naps      - The number of apertures
;     orders    - An array of order numbers
;     start     - Start column
;     stop      - Stop column
;     hdrinfo   - A structure with FITS keywords and values
;     apradius  - The aperture radius of the extraction
;     mode      - A string giving the mode name of the spectra
;     slith_pix - The slit length in pixels
;     slith_arc - The slit length in arcseconds
;     slitw_pix - The slit width in pixels
;     slitw_arc - The slit width in arcseconds
;     version   - Spextool version
;
; OUTUTS:
;     Writes a FITS or text file to disk
;
; KEYWORD PARAMETERS:    
;     BGR       - The background regions of the extraction 
;                 (see extractspec_xs.pro)
;     BGORDER   - The polynomial degree of the background fit
;     TRACEFILE - A string giving the name of the trace file used in 
;                 the extraction (if a trace file was used)
;     ARCIMAGE  - A string giving the name of the arc image      
;     DISP      - An array [nordesr*naps] of wavelength dispersion
;     FITS      - Write a FITS file
;     TEXT      - Write a text file
;     RMS       - An array [naps] of RMS values of the wavelength
;                 solution
;     RES       - The average resolution of the spectra      
;     CANCEL    - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;  
; PROCEDURE:
;
; REVISION HISTORY:
;       written by M. Cushing 9/01/2000
;
pro writespec_xs,spectra,xunits,yunits,fullpath,aimage,sky,flat,naps,orders,$
                 start,stop,hdrinfo,apradius,mode,slith_pix,slith_arc,$
                 slitw_pix,slitw_arc,version,BGR=bgr,BGORDER=bgorder,$
                 TRACEFILE=tracefile,ARCIMAGE=arcimage,DISP=disp,FITS=fits,$
                 TEXT=text,RMS=rms,RES=res,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 19 then begin
    
    print, 'Syntax - writespec_ps,spectra,xunits,yunits,fullpath,aimage,$'
    print,                        sky,flat,naps,orders,start,stop,hdrinfo,$'
    print, '                      apradius,mode,slith_pix,slith_arc,$'
    print, '                      slitw_pix,slitw_arc,version,BGSTART=bgstart,$'
    print, '                      BGWIDTH=bgwidth,BGORDER=bgorder,$'
    print, '                      TRACEFILE=tracefile,ARCIMAGE=arcimage,$'
    print, '                      DISP=disp,FITS=fits,TEXT=text,RMS=rms,$'
    print, '                      RES=res,CANCEL=cancel'

    cancel = 1
    return

endif
zparcheck, 'writespec_xs',spectra, 1, [2,3,4,5], [2,3], 'Spectra'
zparcheck, 'writespec_xs',xunits, 2, 7, 0, 'Xunits'
zparcheck, 'writespec_xs',yunits, 3, 7, 0, 'Yunits'
zparcheck, 'writespec_xs',fullpath, 4, 7, 0, 'Fullpath'
zparcheck, 'writespec_xs',aimage, 5, 7, 0, 'Aimage'
zparcheck, 'writespec_xs',sky, 6, 7, 0, 'Sky'
zparcheck, 'writespec_xs',flat, 7, 7, 0, 'Flat'
zparcheck, 'writespec_xs',naps, 8, [2,3,4,5], 0, 'naps'
zparcheck, 'writespec_xs',orders, 9, [2,3,4,5], [0,1], 'orders'
zparcheck, 'writespec_xs', start, 10, [2,3,4,5], 0, 'Start'
zparcheck, 'writespec_xs', stop, 11, [2,3,4,5], 0, 'Stop'
zparcheck, 'writespec_xs', hdrinfo, 12, 8, [0,1], 'Hdrinfo'
zparcheck, 'writespec_xs', apradius, 13, [2,3,4,5], [0,1], 'Apradius'
zparcheck, 'writespec_xs', mode, 14, 7, 0, 'Mode'
zparcheck, 'writespec_xs', slith_pix, 15, [2,3,4,5], 0, 'Slith_pix'
zparcheck, 'writespec_xs', slith_arc, 16, [2,3,4,5], 0, 'Slith_arc'
zparcheck, 'writespec_xs', slitw_pix, 17, [2,3,4,5], 0, 'Slitw_pix'
zparcheck, 'writespec_xs', slitw_arc, 18, [2,3,4,5], 0, 'Slitw_arc'
zparcheck, 'writespec_xs', version, 19, 7, 0, 'Version'

;  Make hdr with hdrinfo in it.

fxhmake,hdr,spectra

norders  = n_elements(orders)
ntags    = n_tags(hdrinfo)

names = tag_names(hdrinfo)
for i = 0, ntags - 1 do begin

    fxaddpar,hdr,names[i],hdrinfo.(i)

endfor

fxaddpar,hdr,'MODENAME',mode
fxaddpar,hdr,'AIMAGE',aimage
fxaddpar,hdr,'SKY',sky
fxaddpar,hdr,'FLAT',flat

fxaddpar,hdr,'NAPS',naps
fxaddpar,hdr,'NORDERS',norders
fxaddpar,hdr,'ORDERS',strjoin(strcompress(orders,/re),',')
fxaddpar,hdr,'START',start
fxaddpar,hdr,'STOP',stop
fxaddpar,hdr,'SLTH_PIX',slith_pix
fxaddpar,hdr,'SLTH_ARC',slith_arc
fxaddpar,hdr,'SLTW_PIX',slitw_pix
fxaddpar,hdr,'SLTW_ARC',slitw_arc

fxaddpar,hdr,'APRADIUS',strjoin(strcompress(apradius,/re),',')
if n_elements(BGR) ne 0 then fxaddpar,hdr,'BGR',bgr
if n_elements(BGORDER) ne 0 then fxaddpar,hdr,'BGORDER',bgorder

if n_elements(TRACEFILE) ne 0 then fxaddpar,hdr,'TRACEFIL',tracefile
if n_elements(ARCIMAGE) ne 0 then fxaddpar,hdr,'ARCIMAGE',arcimage

fxaddpar,hdr,'XAXIS','Wavelength'
fxaddpar,hdr,'YAXIS','Flux'
fxaddpar,hdr,'XUNITS',xunits
fxaddpar,hdr,'YUNITS',yunits
fxaddpar,hdr,'VERSION',version

if n_elements(RES) ne 0 then fxaddpar,hdr,'RES',res

if n_elements(DISP) ne 0 then begin

    for i = 0,naps-1 do begin

        for j = 0, norders-1 do begin
            
            name = 'DPO'+string(orders[j],format='(i2.2)') + $
              'AP'+string(i+1,format='(i1.1)')
            sxaddpar,hdr,name,disp[i+j*naps]

        endfor

    endfor

endif

history = 'The orders and apertures can be accessed as follows:  If the'+$ 
  ' user desires aperture Y in order X then lambda = [*,0,( X - '+$
  'min(orders))*naps + Y], flux = array[*,1,( X - '+$
  'min(orders))*naps + Y], error = array[*,2,( X - '+$
  'min(orders))*naps + Y]'

if n_elements(RMS) ne 0 then begin

    history =history+'  The RMS in angstroms of the wavelength calibration is '+$
      strjoin(strcompress(rms*10^4,/RE),', ')+' for aperture 1, 2, etc.'

endif

length = strlen(history)
loop = ceil(float(length)/65.)
for i = 0, loop-1 do begin
    
    hist = strmid(history,65*i,65)
    fxaddpar,hdr,'HISTORY',hist
    
endfor

if keyword_set(FITS) then writefits,fullpath+'.fits',spectra,hdr

if keyword_set(TEXT) then begin

    npix = fxpar(hdr,'NAXIS1')
    openw,lun,fullpath+'.txt', /get_lun

    for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]

    for i = 0, npix-1 do begin

        printf, lun,  strjoin( reform(spectra[i,*,*],3*naps*norders),'  ' )

    endfor
    close, lun
    free_lun, lun


endif

end
