; NAME:
;     xcombspec
;
; PURPOSE:
;     To combine spectra together
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xcombspec
;
; INPUTS:
;     None
;
; OUTUTS:
;     Writes a FITS or text file to disk
;
; KEYWORD PARAMETERS:    
;     None
; PROCEDURES CALLED:
;     Requires the Astronomy User's Libray
;     Alot
;
; PROCEDURE:
;     As the spectra are loaded in, they are resampled onto the
;     wavelength array of the first spectrum.  Once they have been
;     loaded, they can be scaled to the median of the stack.  To
;     eliminate bad spectra, click on the order with the bad spectrum.
;     A widget called xselectspec will appear and the spectrum can 
;     be eliminated from the stack by unclicking its number.  If the 
;     spectrum is bad for ever order, then clicking All Orders
;     eliminates it from all of the orders.
;
;     If an unweighted mean or median combination is chosen, then 
;     the resulting error spectrum is computed via sigma_mu =
;     sigma/sqrt(N) where N is the number of spectra and sigma is the 
;     standard deviation of the data at a particular wavelength.  
;     If a median is chosen, then sigma_med = MAD/sqrt(N) (see
;     medcomb.pro).
;
; MODIFICATION HISTORY:
;     ? - Written by M. Cushing, Institute for Astronomy, UH
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xcombspec_event, event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue
widget_control, /HOURGLASS

case uvalue of

    'Aperture': begin

        state.r.ap = event.index
        xcombspec_plotspec,state

    end

    'Combine Apertures': state.r.combineaps = event.value

    'Combine Spectra': xcombspec_combinespec,state

    'Combination Statistic': begin

        state.r.combinestat = event.value
        s = (state.r.combinestat eq 'Median') ? 0:1
        widget_control, state.w.weights, SENSITIVE=s 

    end
    'File Name': begin

        path =dialog_pickfile(DIALOG_PARENT=state.w.xcombspec_base,$
                              /MUST_EXIST)

        if path ne '' then begin

            widget_control,state.w.filename_fld[1],SET_VALUE = path

        endif

    end

    'Input Mode': begin

        widget_control, state.w.manual_base,map=0
        widget_control, state.w.file_base,map=0
        state.r.inputmode = event.value
        if event.value eq 'File' then widget_control, state.w.file_base,/MAP
        if event.value eq 'Manual' then widget_control, state.w.manual_base,$
          /MAP


    end

    'Input Prefix': setfocus, state.w.specfiles_fld

    'Load Spectra': xcombspec_loadspec,state

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select

    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xcombspec_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xcombspec_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld

        endif
    end

    'Readmode': begin

        widget_control, state.w.inprefix_fld[0],  SENSITIVE=0
        if event.value eq 'Filename' then begin
            
            state.r.filereadmode = event.value
            widget_control, state.w.inprefix_fld[0], SENSITIVE=0
            setfocus,state.w.specfiles_fld
            
        endif else begin
            
            state.r.filereadmode = event.value
            widget_control, state.w.inprefix_fld[0], /SENSITIVE
            setfocus,state.w.inprefix_fld

        endelse

    end

    'Quit': begin

        widget_control, event.top, /destroy
        goto, getout
        
    end

    'Scale to Median': begin

        if strtrim(event.value,2) eq 'Yes' then begin
            
            state.r.scalespec = 'Yes'
            xcombspec_scalespec,state 

        endif else begin

            *state.d.wspec = *state.d.ospec
            xcombspec_plotspec,state
            state.r.scalespec = 'No'

        endelse

    end
    'Spectra Files Button': begin
                
        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        if path ne '' then path = cpath(path,WIDGET_ID=state.w.xcombspec_base,$
                                        CANCEL=cancel)

        fullpath = dialog_pickfile(DIALOG_PARENT=state.w.xcombspec_base,$
                                   PATH=path,/MUST_EXIST,FILTER='*.fits',$
                                   /MULTIPLE_FILES)
        
        case (size(fullpath))[1] of 

            1: begin

                if fullpath[0] ne '' then begin
                    
                    widget_control,state.w.specfiles_fld[1],$
                      set_value = strmid(fullpath[0],$
                                         strpos(fullpath,'/',/REVERSE_S)+1)
                    setfocus, state.w.outfile_fld
                    
                endif

            end

            else: begin

                for i =0,(size(fullpath))[1]-1 do begin
 
                    tmp = strmid(fullpath[i],strpos(fullpath[i],'/',$
                                                    /REVERSE_S)+1)
                    arr = (i eq 0) ? tmp:[arr,tmp]

                endfor
                widget_control,state.w.specfiles_fld[1],$
                  set_value = strjoin(arr,',',/SINGLE)
                setfocus, state.w.outfile_fld

            end

        endcase

    end

    'Spectra Files Field': setfocus,state.w.outfile_fld

    'Weights': state.r.weights = event.value

    else:


endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xcombspec_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
pro xcombspec_resize,event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

widget_control, state.w.xcombspec_base, tlb_get_size = size

state.p.plotwinsize[0]=size[0]-state.p.buffer[0]
state.p.plotwinsize[1]=size[1]-state.p.buffer[1]

widget_control, state.w.plotwin, draw_xsize=state.p.plotwinsize[0]
widget_control, state.w.plotwin, draw_ysize=state.p.plotwinsize[1]

xcombspec_plotspec,state

widget_control, state.w.xcombspec_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xcombspec_selectspec,event

if not event.press then goto, cont

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

spc = *state.d.wspec
mask = *state.r.mask

idx = floor(event.y/float(state.p.plotwinsize[1])*state.r.norders)

xselectspec,reform(spc[state.r.ap].(idx)[*,0,0]),$
  reform(spc[state.r.ap].(idx)[*,1,*]),omask,$
  GROUP_LEADER=state.w.xcombspec_base,$
  IMASK=reform(mask[state.r.ap].(idx)),ALLORDERS=allorders,$
  yrange=(*state.p.plotrange)[*,idx]

if allorders then begin

    for i = 0, state.r.norders-1 do mask[state.r.ap].(i) = omask

endif else mask[state.r.ap].(idx) = omask

*state.r.mask = mask

xcombspec_plotspec,state

widget_control, event.top, SET_UVALUE=state, /NO_COPY
cont:

spc = 0B
end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xcombspec_cleanup,xcombspec_base

widget_control, xcombspec_base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin
    
    ptr_free, state.r.files
    ptr_free, state.r.mask
    ptr_free, state.r.orders
    ptr_free, state.p.plotrange
    ptr_free, state.d.ospec
    
    ptr_free, state.d.wspec
    ptr_free, state.d.hdrinfo
    
endif 
state = 0B

end
;
;******************************************************************************
;
pro xcombspec_combinespec,state

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

outname = cfld(state.w.outfile_fld,7,CANCEL=cancel,/EMPTY)
if cancel then return

spc = *state.d.wspec
mask = *state.r.mask

sspec = (state.r.combineaps eq 'Yes') ? $
  fltarr(state.r.npix,3,state.r.norders):$
  fltarr(state.r.npix,3,state.r.norders*state.r.naps)

naps = (state.r.combineaps eq 'No') ? state.r.naps:1.

for i = 0, naps-1 do begin

    for j = 0, state.r.norders-1 do begin

        z = where(mask[i].(j) eq 1)
        if state.r.weights eq 'Yes' then var = reform((spc[i].(j)[*,2,z])^2)
        case state.r.combinestat of 
            
            'Robust Mean': robustmeancomb,reform(spc[i].(j)[*,1,z]),4,mean,$
              mvar,DATAVAR=var,CANCEL=cancel
            
            'Mean': meancomb,reform(spc[i].(j)[*,1,z]),mean,mvar,$
              DATAVAR=var,CANCEL=cancel
            
            'Median': medcomb,reform(spc[i].(j)[*,1,z]),mean,mvar,$
              CANCEL=cancel
            
        endcase
        
        sspec[*,0,j*naps+i] = reform(spc[i].(j)[*,0,0])
        sspec[*,1,j*naps+i] = mean
        sspec[*,2,j*naps+i] = sqrt(mvar)
       
    endfor

endfor

;  Make new hdr.

avehdr = avehdrs(*state.d.hdrinfo,TIME_OBS=state.r.time,$
                 POSANGLE=state.r.posangle,HA=state.r.ha,$
                 AIRMASS=state.r.airmass,CANCEL=cancel)

avehdr.NAPS = (state.r.combineaps eq 'Yes') ? 1:state.r.naps

weighted = (state.r.weights eq 'Yes') ? 'Weighted ':''

history='The spectra '+strjoin(*state.r.files,', ')+' were combined using a '+$
  weighted+state.r.combinestat+' to create these spectra.'

history = (state.r.scalespec eq 'Yes') ? $
  history+'  The spectra were first scaled to the spectrum with the highest '+$
  'flux level.': history

fxhmake,newhdr,sspec

ntags    = n_tags(avehdr)
names = tag_names(avehdr)
for i = 0, ntags - 1 do begin

    if names[i] eq 'HISTORY' then begin

        for j = 0, ceil(float(strlen(avehdr.(i)))/70.)-1 do begin
            
            hist = strmid(avehdr.(i),70*j,70)
            fxaddpar,newhdr,'HISTORY',hist
            
        endfor
       
    endif else fxaddpar,newhdr,names[i],avehdr.(i)

endfor

for i = 0, ceil(float(strlen(history))/70.)-1 do begin
    
    hist = strmid(history,70*i,70)
    fxaddpar,newhdr,'HISTORY',hist
    
endfor

xvspec,sspec,state.r.norders,avehdr.NAPS,$
  ORDERS=long( strsplit(avehdr.ORDERS,',',/extrac) ),XUNITS=xunits,$
  YUNITS=yunits,FILE=outname+'.fits',HDR=newhdr

if state.r.fitsoutput then writefits,path+outname+'.fits',sspec,newhdr

if state.r.textoutput then begin $

    t = (state.r.combineaps eq 'Yes') ? 1:state.r.naps
    npix = fxpar(newhdr,'NAXIS1')

    openw,lun,path+outname+'.txt', /get_lun

    for i = 0, n_elements(newhdr)-1 do printf, lun, newhdr[i]

    for i = 0, npix[0]-1 do begin

        printf, lun,  strjoin( reform(sspec[i,*,*],3*t*state.r.norders),'  ' )

    endfor
    close, lun
    free_lun, lun

endif

end
;
;******************************************************************************
;
pro xcombspec_loadspec,state

if state.r.inputmode eq 'Manual' then begin
    
;  Get path and file names
    
    path = cfld(state.w.path_fld,7,CANCEL=cancel)
    if cancel then return

    files = cfld(state.w.specfiles_fld, 7, CANCEL=cancel, /EMPTY)
    if cancel then return
    
;  Construct array of full file names
    
    index    = (state.r.filereadmode eq 'Index') ? 1:0
    filename = (state.r.filereadmode eq 'Filename') ? 1:0
    if index then prefix = cfld(state.w.inprefix_fld,7,/EMPTY,CANCEL=cancel)
    if cancel then return
    
    files = fsextract(files,INDEX=index,FILENAME=filename,CANCEL=cancel)
    if cancel then return
    
    fullpaths = mkfullpath(path,files,INDEX=index,FILENAME=filename,NI=4,$
                           PREFIX=prefix,SUFFIX='.fits',$
                           WIDGET_ID=state.w.xcombspec_base,/EXIST,$
                           CANCEL=cancel)
    if cancel then setfocus,state.w.specfiles_fld
    if cancel then return
    
endif else begin

    filename = cfld(state.w.filename_fld,7,CANCEL=cancel)
    if cancel then return    
    nl = numlines(filename)
    fullpaths = strarr(nl)
    openr,lun, filename,/get_lun
    line = ' '
    for i = 0, nl-1 do begin

        readf, lun, line
        fullpaths[i] = strtrim(line,2)

    endfor
    free_lun, lun
    path = ''
    
endelse

junk   = readfits(fullpaths[0],hdr)

state.r.npix     = fxpar(hdr,'NAXIS1')
state.r.norders  = fxpar(hdr,'NORDERS')
state.r.naps     = fxpar(hdr,'NAPS')
state.p.xunits   = fxpar(hdr,'XUNITS')
state.p.yunits   = fxpar(hdr,'YUNITS')
*state.r.orders  = long( strsplit( fxpar(hdr,'ORDERS'), ',', /extract) )
state.r.nfiles   = n_elements(fullpaths)

;  Combining apertures?

if state.r.combineaps eq 'No' then begin

    widget_control, state.w.aperture_dl, /SENSITIVE
    value = 'Aperture '+strcompress(indgen(state.r.naps)+1, /re)
    widget_control, state.w.aperture_dl, set_value=value

endif else begin

    widget_control, state.w.aperture_dl, SENSITIVE=0
    state.r.ap = 0

endelse

;  Create filename string for FITS header later

for i = 0,state.r.nfiles-1 do begin

    file = strmid(fullpaths[i],strpos(fullpaths[i],'/',/REVERSE_S)+1)
    sfile = (i eq 0) ? file:[sfile,file]

endfor
*state.r.files = sfile

;  Load data into memory

hdrinfo = replicate(gethdrinfo(hdr),state.r.nfiles)
names = string(findgen(state.r.nfiles))

for i = 0, state.r.nfiles-1 do begin
    
    data    = readfits(fullpaths[i],hdr)
    bigspec = (i eq 0) ? data:[[[bigspec]],[[data]]]
    copy_struct_inx,gethdrinfo(hdr),hdrinfo,index_to=i

endfor

;  Set up data structure.

t = (state.r.combineaps eq 'Yes') ? state.r.naps:1

for i = 0, state.r.norders-1 do begin

    key = 'Order'+string(i,format='(i2.2)')
    spc = (i eq 0) ? create_struct(key,fltarr(state.r.npix,3,$
                                              t*state.r.nfiles)):$
      create_struct(spc,key,fltarr(state.r.npix,3,t*state.r.nfiles))
    pmask = (i eq 0) ? create_struct(key,intarr(t*state.r.nfiles)+1):$
      create_struct(pmask,key,intarr(t*state.r.nfiles)+1)

endfor

if state.r.combineaps eq 'No' then begin

    spc = replicate(spc,state.r.naps)
    pmask = replicate(pmask,state.r.naps)

endif

;  Now resample if necessary and load into structure.


if state.p.xunits ne 'Pixels' then begin

    for i = 0, state.r.norders-1 do begin
        
        if state.r.combineaps eq 'Yes' then begin
            
            mask  = intarr(state.r.norders*state.r.naps)
            mask[i*state.r.naps:i*state.r.naps+(state.r.naps-1)] = 1
            smask = replicas(mask,state.r.nfiles)
            z     = where(smask eq 1,count)

            for k = 0, count-1 do begin

                interpspec,reform(bigspec[*,0,z[k]]),$
                  reform(bigspec[*,1,z[k]]),$
                  reform(bigspec[*,0,z[0]]),result,error,$
                  YAERROR=reform(bigspec[*,2,z[k]])

                spc[0].(i)[*,0,k] = reform(bigspec[*,0,z[0]])
                spc[0].(i)[*,1,k] = result
                spc[0].(i)[*,2,k] = error
                
            endfor

        endif else begin
            
            for j = 0, state.r.naps-1 do begin

                mask  = intarr(state.r.norders*state.r.naps)
                mask[i*state.r.naps+j] = 1
                smask = replicas(mask,state.r.nfiles)
                z     = where(smask eq 1,count)
                for k = 0, count-1 do begin

                    interpspec,reform(bigspec[*,0,z[k]]),$
                      reform(bigspec[*,1,z[k]]),$
                      reform(bigspec[*,0,z[0]]),result,error,$
                      YAERROR=reform(bigspec[*,2,z[k]])

                    spc[j].(i)[*,0,k] = reform(bigspec[*,0,z[0]])
                    spc[j].(i)[*,1,k] = result
                    spc[j].(i)[*,2,k] = error
                    
                endfor
                
            endfor
            
        endelse

    endfor

endif

*state.d.ospec    = spc
*state.d.wspec    = spc
*state.r.mask     = pmask
*state.d.hdrinfo  = hdrinfo

;  Set' scale to median' to no.

widget_control, state.w.medscale, set_value=1

xcombspec_plotspec,state

end
;
;******************************************************************************
;
pro xcombspec_plotspec,state

wset, state.p.plotwin_wid
erase

spc                = *state.d.wspec
mask               = *state.r.mask
*state.p.plotrange = fltarr(2,state.r.norders)

!p.multi[0] = state.r.norders
!p.multi[2] = state.r.norders

t = (state.r.combineaps eq 'Yes') ? state.r.naps:1
for i = 0, state.r.norders-1 do begin

    j     = state.r.norders-1-i
    title = 'Order '+strcompress((*state.r.orders)[j], /re)

;  Get plot range.

    med = fltarr(state.r.npix)
    for k = 0, state.r.npix-1 do med[k] = median(spc[state.r.ap].(j)[k,1,*],$
                                                 /EVEN)
    ymax = max(med,/NAN,min=ymin)
    xmax = max(spc[state.r.ap].(j)[*,0,*],min=xmin)
    (*state.p.plotrange)[*,j] = [ymin,ymax]


    plot, spc[state.r.ap].(j)[*,0,*],spc[state.r.ap].(j)[*,1,*],/xsty,/ysty,$
      xrange=[xmin,xmax],yrange=[1.2*ymin,1.5*ymax],/NODATA,charsize=2,$
      title=title

    for k = 0, t*state.r.nfiles-1 do begin

        if mask[state.r.ap].(j)[k] eq 1 then $
        oplot,spc[state.r.ap].(j)[*,0,k],spc[state.r.ap].(j)[*,1,k],$
          color=state.r.colors[k],linestyle=state.r.lines[k]

    endfor

endfor
!p.multi=0
spc = 0B

end
;
;******************************************************************************
;
pro xcombspec_scalespec,state

spc  = *state.d.wspec
mask = *state.r.mask
naps = (state.r.combineaps eq 'No') ? state.r.naps:1.
idxs = intarr(naps)

;  Get stack number of spectrum with max flux

for i = 0, naps-1 do begin

    sidx = 0
    for j = 0, state.r.norders-1 do begin

        z = where(mask[i].(j) eq 1)
        s = reform(spc[i].(j)[*,1,z])
        e = reform(spc[i].(j)[*,2,z])
        scalespec,s,e,/MAX,IDX=idx,CANCEL=cancel
        if cancel then return
        sidx = sidx+idx

    endfor
    idxs[i] = round(sidx/float(state.r.norders))

endfor

;  Scale spectra

spc  = *state.d.wspec
for i = 0, naps-1 do begin

    for j = 0, state.r.norders-1 do begin

        z = where(mask[i].(j) eq 1)
        s = reform(spc[i].(j)[*,1,z])
        e = reform(spc[i].(j)[*,2,z])
        scalespec,s,e,IDX=idxs[i],CANCEL=cancel
;        scalespec,s,e,/MEDIAN,CANCEL=cancel
        if cancel then return

        spc[i].(j)[*,1,z] = s
        spc[i].(j)[*,2,z] = e        

    endfor

endfor

*state.d.wspec = spc
xcombspec_plotspec,state

end
;
;******************************************************************************
;
pro xcombspec,instrfile

mkct

;  Determine the instrument in use, default is SpeX.

if n_elements(instrfile) eq 0 then instrfile = 'SpeX.dat'

last   = strpos(!path,'Spextool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
result = strmid(!path,first+1,last-first+7)

path = cpath(result,CANCEL=cancel)
if cancel then return

readinstrfile,path+'data/'+instrfile,instr,irafname,gain,readnoise,itime,$
  coadds,ndrs,time,posangle,ha,airmass,nint,bdpxmk,keywords,$
  ncols,nrows,CANCEL=cancel

;  Set the fonts

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {aperture_dl:0L,$
     manual_base:0L,$
     medscale:0L,$
     path_fld:[0L,0L],$
     file_base:0L,$
     filename_fld:[0L,0L],$
     inprefix_fld:[0L,0L],$
     outfile_fld:[0L,0L],$
     plotwin:0L,$
     weights:0L,$
     specfiles_fld:[0L,0L],$
     xcombspec_base:0L}

r = {airmass:airmass,$
     ap:0,$
     ha:ha,$
     colors:[1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6,7,8,9,10,$
             1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6,7,8,9,10],$
     combineaps:'No',$
     combinestat:'Robust Mean',$
     files:ptr_new(fltarr(2)),$
     filereadmode:'Index',$
     fitsoutput:1,$
     ifileformat:'FITS',$
     inputmode:'Manual',$
     lines:[0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,$
            3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5],$
     mask:ptr_new(intarr(2)),$
     naps:0,$
     nfiles:0,$
     norders:0,$
     npix:0,$
     orders:ptr_new(fltarr(2)),$     
     posangle:posangle,$
     scalespec:'No',$
     textoutput:0,$
     time:time,$
     weights:'Yes'}

p = {buffer:[0,0],$
     plotrange:ptr_new(fltarr(2)),$
     plotwin_wid:0,$
     plotwinsize:[512,600],$
     xunits:'',$
     yunits:''}

d = {ospec:ptr_new(fltarr(2)),$
     wspec:ptr_new(fltarr(2)),$
     hdrinfo:ptr_new(strarr(2))}

state = {w:w,r:r,d:d,p:p}

;  Build the widget.

state.w.xcombspec_base = widget_base(title='Combine Spectra', $
                                     /COLUMN,$
                                     /TLB_SIZE_EVENTS)

   quit_button = widget_button(state.w.xcombspec_base,$
                               FONT=buttonfont,$
                               VALUE='Quit',$
                               EVENT_PRO='xcombspec_event',$
                               UVALUE='Quit')

   row_base = widget_base(state.w.xcombspec_base,$
                          /ROW)

      col1_base = widget_base(row_base,$
                              /COLUMN)

         box1_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 EVENT_PRO='xcombspec_event',$
                                 /FRAME)

            bg = cw_bgroup(box1_base,$
                           ['Manual','File'],$
                           /ROW,$
                           LABEL_LEFT='Input Mode:',$
                           /RETURN_NAME,$
                           /NO_RELEASE,$
                           UVALUE='Input Mode',$
                           FONT=buttonfont,$
                           /EXCLUSIVE,$
                           SET_VALUE=0)

            blank = widget_base(box1_base,$
                                /BASE_ALIGN_CENTER)

               state.w.manual_base = widget_base(blank,$
                                                 /COLUMN,$
                                                 /BASE_ALIGN_CENTER)

                  row = widget_base(state.w.manual_base,$
                                    /ROW,$
                                    /BASE_ALIGN_CENTER)
                  
                     button = widget_button(row,$
                                            FONT=buttonfont,$
                                            VALUE='Path',$
                                            UVALUE='Path Button',$
                                            EVENT_PRO='xcombspec_event')
                     
                     field = coyote_field2(row,$
                                           LABELFONT=buttonfont,$
                                           FIELDFONT=textfont,$
                                           TITLE=':',$
                                           UVALUE='Path Field',$
                                           XSIZE=25,$
                                           EVENT_PRO='xcombspec_event',$
                                           /CR_ONLY,$
                                           TEXTID=textid)
                     state.w.path_fld = [field,textid]               
               



                  bg = cw_bgroup(state.w.manual_base,$
                                 ['Filename','Index'],$
                                 /ROW,$
                                 LABEL_LEFT='File Read Mode:',$
                                 /RETURN_NAME,$
                                 /NO_RELEASE,$
                                 UVALUE='Readmode',$
                                 FONT=buttonfont,$
                                 /EXCLUSIVE,$
                                 SET_VALUE=1)
            
                  field = coyote_field2(state.w.manual_base,$
                                        labelfont=buttonfont,$
                                        fieldfont=textfont,$
                                        title='Input Prefix:',$
                                        uval = 'Input Prefix',$
                                        xsize=25,$
                                        event_pro = 'xcombspec_event',$
                                        /cr_only,$
                                        textid=textid)
                  state.w.inprefix_fld = [field,textid]
                  
                  row = widget_base(state.w.manual_base,$
                                    /ROW,$
                                    /BASE_ALIGN_CENTER)
                  
                     button = widget_button(row,$
                                            FONT=buttonfont,$
                                            VALUE='Spectra Files',$
                                            UVALUE='Spectra Files Button',$
                                            EVENT_PRO='xcombspec_event')
                     
                     field = coyote_field2(row,$
                                           LABELFONT=buttonfont,$
                                           FIELDFONT=textfont,$
                                           TITLE=':',$
                                           UVALUE='Spectra Files Field',$
                                           XSIZE=25,$
                                           EVENT_PRO='xcombspec_event',$
                                           /CR_ONLY,$
                                           TEXTID=textid)
                     state.w.specfiles_fld = [field,textid]
                     
               state.w.file_base = widget_base(blank,$
                                               /COLUMN,$
                                               /BASE_ALIGN_CENTER)

                  row = widget_base(state.w.file_base,$
                                    /ROW,$
                                    /BASE_ALIGN_CENTER)
                  
                     button = widget_button(row,$
                                            FONT=buttonfont,$
                                            VALUE='File Name',$
                                            UVALUE='File Name',$
                                            EVENT_PRO='xcombspec_event')
                     
                     field = coyote_field2(row,$
                                           LABELFONT=buttonfont,$
                                           FIELDFONT=textfont,$
                                           TITLE=':',$
                                           UVALUE='File Name Field',$
                                           XSIZE=25,$
                                           EVENT_PRO='xcombspec_event',$
                                           /CR_ONLY,$
                                           TEXTID=textid)
                     state.w.filename_fld = [field,textid]
                     
            combineap_bg = cw_bgroup(box1_base,$
                                     FONT=buttonfont,$
                                     ['Yes','No'],$
                                     /ROW,$
                                     /RETURN_NAME,$
                                     /NO_RELEASE,$
                                     /EXCLUSIVE,$
                                     LABEL_LEFT='Combine Apertures:',$
                                     UVALUE='Combine Apertures',$
                                     SET_VALUE=1)


            load = widget_button(box1_base,$
                                 FONT=buttonfont,$
                                 VALUE='Load Spectra',$
                                 UVALUE='Load Spectra')
            


         box2_base = widget_base(col1_base,$
                                 EVENT_PRO='xcombspec_event',$
                                 /COLUMN,$
                                 /FRAME)

            state.w.medscale = cw_bgroup(box2_base,$
                                         FONT=buttonfont,$
                                         ['Yes','No'],$
                                         /ROW,$
                                         /RETURN_NAME,$
                                         /NO_RELEASE,$
                                         /EXCLUSIVE,$
                                         LABEL_LEFT='Scale to Max:',$
                                         UVALUE='Scale to Median',$
                                         SET_VALUE=1)
            
            combine_bg = cw_bgroup(box2_base,$
                                   FONT=buttonfont,$
                                   ['Robust Mean','Mean','Median'],$
                                   /ROW,$
                                   /RETURN_NAME,$
                                   /NO_RELEASE,$
                                   /EXCLUSIVE,$
                                   LABEL_TOP='Combination Statistic:',$
                                   UVALUE='Combination Statistic',$
                                   SET_VALUE=0)

            state.w.weights = cw_bgroup(box2_base,$
                                        FONT=buttonfont,$
                                        ['Yes','No'],$
                                        /ROW,$
                                        /RETURN_NAME,$
                                        /NO_RELEASE,$
                                        /EXCLUSIVE,$
                                        LABEL_LEFT='Weighted Mean:',$
                                        UVALUE='Weights',$
                                        SET_VALUE=0)


         box3_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /BASE_ALIGN_CENTER,$
                                 EVENT_PRO='xcombspec_event',$
                                 /FRAME)

            outformat_bg = cw_bgroup(box3_base,$
                                     FONT=buttonfont,$
                                     ['FITS','Text'],$
                                     /ROW,$
                                     /RETURN_NAME,$
                                     /NONEXCLUSIVE,$
                                     LABEL_LEFT='Output Format:',$
                                     UVALUE='Output Format',$
                                     SET_VALUE=[1,0])
         
            field = coyote_field2(box3_base,$
                                  LABELFONT=buttonfont,$
                                  FIELDFONT=textfont,$
                                  TITLE='Output File:',$
                                  UVALUE='Output File',$
                                  XSIZE=25,$
                                  EVENT_PRO='xcombspec_event',$
                                  /CR_ONLY,$
                                  TEXTID=textid)
            state.w.outfile_fld = [field,textid]

         combine_button = widget_button(col1_base,$
                                        EVENT_PRO='xcombspec_event',$
                                        FONT=buttonfont,$
                                        VALUE='Combine Spectra',$
                                        UVALUE='Combine Spectra')

         col2_base = widget_base(row_base,$
                                 /COLUMN)

            state.w.aperture_dl = widget_droplist(col2_base,$
                                                  EVENT_PRO='xcombspec_event',$
                                                  FONT=buttonfont,$
                                                  VALUE='Aperture 1',$
                                                  UVALUE='Aperture')   
            
            state.w.plotwin = widget_draw(col2_base,$
                                          XSIZE=state.p.plotwinsize[0],$
                                          YSIZE=state.p.plotwinsize[1],$
                                          UVALUE='Plot Window',$
                                          /BUTTON_EVENTS,$
                                          EVENT_PRO='xcombspec_selectspec')


; Get things running.  Center the widget using the Fanning routine.

centertlb,state.w.xcombspec_base

widget_control, state.w.xcombspec_base, /realize

;  Get plotwin ids

widget_control, state.w.plotwin, get_value = x
state.p.plotwin_wid = x

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xcombspec', $
  state.w.xcombspec_base, $
  /No_Block,$
  EVENT_HANDLER='xcombspec_resize',$
  cleanup = 'xcombspec_cleanup'

;  Get sizes of things now for resizing

widget_control, state.w.xcombspec_base, tlb_get_size = result

widget_geom = widget_info(state.w.xcombspec_base, /geometry)
window_geom = widget_info(state.w.plotwin,  /geometry)

state.p.buffer[0] = result[0] + 2.*widget_geom.margin - $
  window_geom.xsize
state.p.buffer[1] = result[1] + 2.*widget_geom.margin - $
  window_geom.ysize



; Put state variable into the user value of the top level base.

widget_control, state.w.xcombspec_base, set_uvalue=state, /no_copy

end






