; NAME:
;     xplotprofiles
;
; PURPOSE:
;     To plot the spatial profiles of a X-dispersed image
;       
; CATEGORY:
;     Plotting
;
; CALLING SEQUENCE:
;     xplotprofiles,data,orders,doorders,slith_arc,POSITIONS=positions,$
;                   MASK=mask,GROUP_LEADER=group_leader,BGFIT=bgfit,$
;                   CANCEL=cancel
;
; INPUTS:
;     data      - An array [norders] of structures where 
;                 struct = [[pixels],[arcseconds],[data],[SG smoothed data],$
;                           [good/bad mask]]
;                 In the good/bad mask, good = 1, bad = 0
;     orders    - An array [norders] of order numbers
;     doorders  - An array [norders] where 1 means plot POSITIONS and
;                 MASK while 0 mean do not
;     slith_arc - Slit length in arcseconds
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     POSITIONS    - An array [naps,norders] of aperture positions
;     MASK         - An array [norders] of structures {x_values,mask}
;                    giving the mask.  (See mkmask_ps.pro)
;     GROUP_LEADER - The widget ID of an existing widget that serves
;                    as "group leader" for the newly-created widget. 
;     CANCEL       - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     Alot
;
; PROCEDURE:
;     The resizable widget shows the superprofiles
;
; MODIFICATION HISTORY:
;     ? - Written by M. Cushing, Institute for Astronomy, UH
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xplotprofiles_initcommon


;  Build the structures which will hold the important info.
;  w - contains info pertaining to widget operations.

common xplotprofiles_state, state

w = {max_fld:[0L,0L],$
     message:0L,$
     min_fld:[0L,0L],$
     order_dl:0L,$
     plotwin:0L,$
     range_base:0L,$
     xplotprofiles_base:0L}

p = {buffer:[0L,0L],$
     doorders:ptr_new(intarr(1)),$
     bgfit:-1,$
     fix:0,$
     mask:0L,$
     nodata:0,$
     npeaks:0L,$
     pixpp:((get_screen_size())[1]-150)/float(6),$
     plotwin_wid:0L,$
     pixmap_wid:0L,$
     ysize:0,$
     positions:0,$
     pscale:!p,$
     ranges:ptr_new(fltarr(2)),$
     rangeorder:0,$
     scrollsize:[600L,(get_screen_size())[1]-150],$
     slith:0.,$
     xscale:!x,$
     yscale:!y}

d = {mask:ptr_new(fltarr(1)),$
     orders:ptr_new(intarr(2)),$
     norders:0,$
     positions:ptr_new(fltarr(2)),$
     profiles:ptr_new(fltarr(2,2)),$
     x_arcs:ptr_new(fltarr(2))}

state = {w:w,p:p,d:d}

end
;
;******************************************************************************
;
pro xplotprofiles_cleanup,event

common xplotprofiles_state

ptr_free, state.p.doorders
ptr_free, state.d.mask
ptr_free, state.d.orders
ptr_free, state.d.positions
ptr_free, state.d.profiles
ptr_free, state.d.x_arcs

state = 0B

end
;
;******************************************************************************
;
pro xplotprofiles_minmax

common xplotprofiles_state

widget_control, state.w.min_fld[1], set_value=strcompress($
  (*state.p.ranges)[0,state.p.rangeorder], /re)

widget_control, state.w.max_fld[1], set_value=strcompress($
  (*state.p.ranges)[1,state.p.rangeorder], /re)

end
;
;******************************************************************************
;
pro xplotprofiles_plot,PS=ps

common xplotprofiles_state

if keyword_set(PS) then begin

    xplotprofiles_plotprofiles

endif else begin

    wset, state.p.plotwin_wid
    erase
    xplotprofiles_plotprofiles
    wset, state.p.pixmap_wid
    erase
    xplotprofiles_plotprofiles

endelse
!p.multi=0


end
;
;******************************************************************************
;
pro xplotprofiles_plotprofiles

common xplotprofiles_state

!p.multi[2] = state.d.norders
!p.multi[0] = state.d.norders

if not state.p.fix then *state.p.ranges = fltarr(2,state.d.norders)

for i = 0, state.d.norders-1 do begin

    j = state.d.norders-1-i
    prof = (*state.d.profiles).(j)

;  Get plot range.

    if state.p.fix then begin

        ymin = (*state.p.ranges)[0,i]
        ymax = (*state.p.ranges)[1,i]

    endif else begin

        ymin = min(prof[*,3],max=ymax)
        del  = ymax-ymin
        ymin = ymin-0.1*del
        ymax = ymax+0.1*del
        
    endelse
    title = '!5Order '+strcompress((*state.d.orders)[j], /re)
    plot,prof[*,1],prof[*,2],charsize=2,xrange=[0,state.p.slith],$
      xticks=tics,xticklen = 0.06,xminor=5,xtitle='!5arcsec',$
      title=title,ytitle='!5Fractional Flux',/ysty,yrange=[ymin,ymax],/NODATA

    if not state.p.fix then (*state.p.ranges)[*,i] = !y.crange

    if not state.p.nodata then oplot, prof[*,1],prof[*,2],psym=3
    oplot, prof[*,1],prof[*,3],color=pscolor((state.p.nodata eq 1) ? 1:2)

    if i eq 0 then begin

        state.p.pscale = !p
        state.p.xscale = !x
        state.p.yscale = !y

    endif

    if state.p.positions then begin

        if (*state.p.doorders)[j] then begin
            
            for k = 0, state.p.npeaks-1 do $
              plots,[(*state.d.positions)[k,j],$
                     (*state.d.positions)[k,j]],!y.crange,color=7
            
        endif

    endif
    if state.p.mask then begin

        mask = (*state.d.mask).(j)
        if (*state.p.doorders)[j] then begin
                
            z = where(mask[*,1] eq -1, count)
            if count eq 0 then goto, cont
            bgmask = fltarr(n_elements((mask[*,0])))
            bgmask[*] = !values.f_nan
            bgmask[z] = prof[z,3]
            oplot, mask[*,0],bgmask,psym=10,color=2
            if state.p.bgfit ne -1 then begin

                coeff = poly_fit(mask[z,0],bgmask[z],state.p.bgfit)
                oplot,mask[*,0],poly(mask[*,0],coeff),color=6,linestyle=1

            endif
            cont:
            for k = 0, state.p.npeaks-1 do begin

                z= where(mask[*,1] gt float(k) and $
                         mask[*,1] le float(k+1),count)
                apmask = fltarr(n_elements(mask[*,0]))
                apmask[*] = !values.f_nan
                apmask[z] = prof[z,3]
                oplot, mask[*,0],apmask,psym=10,color=3
                plots, [mask[z[0],0],mask[z[0],0]],$
                  !y.crange,linestyle=1,color=3
                plots, [mask[z[count-1],0],mask[z[count-1],0]],$
                  !y.crange,linestyle=1,color=3

            endfor
            
        endif

    endif

endfor
!p.multi=0


end
;
;******************************************************************************
;
pro xplotprofiles_range

common xplotprofiles_state

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not xregistered('xplotprofiles_range') then begin

    state.w.range_base = widget_base(GROUP_LEADER=state.w.xplotprofiles_base, $
                                     /COLUMN, $
                                     TITLE='Plot Range')

       value = 'Order '+strcompress( reverse((*state.d.orders)), /re)
       state.w.order_dl = widget_droplist(state.w.range_base,$
                                          font=buttonfont,$
                                          value=value,$
                                          /dynamic_resize,$
                                       event_pro='xplotprofiles_button_event',$
                                          uvalue='Order')  

       min = coyote_field2(state.w.range_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='Min:',$
                           uvalue='Min',$
                           xsize=10,$
                           /cr_only,$
                           event_pro = 'xplotprofiles_button_event',$
                           textid=textid)
       state.w.min_fld = [min,textid]

       max = coyote_field2(state.w.range_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='Max:',$
                           uvalue='Max',$
                           xsize=10,$
                           /cr_only,$
                           event_pro = 'xplotprofiles_button_event',$
                           textid=textid)
       state.w.max_fld = [max,textid]

       kill = widget_button(state.w.range_base,$
                            value='Quit',$
                            uvalue='Quit Range',$
                            event_pro = 'xplotprofiles_button_event',$
                            font=buttonfont)

       widget_control, state.w.range_base, /Realize
       
; Start the Event Loop. This will be a non-blocking program.
       
       XManager, 'xplotprofiles_range', $
         state.w.range_base, $
         /No_Block
         event_handler='xplotprofiles_buttons'

       xplotprofiles_minmax
       state.p.fix = 1

endif else begin

value = 'Order '+strcompress( reverse((*state.d.orders)), /re)
widget_control, state.w.order_dl, set_value=value
state.p.rangeorder = 0
xplotprofiles_minmax

endelse




end
;
;******************************************************************************
;
pro xplotprofiles_startup,GROUP_LEADER=group_leader

common xplotprofiles_state

buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

;  Make the widget
    
state.w.xplotprofiles_base = widget_base(title='Plot Profiles',$
                                         /column,$
                                         group_leader=group_leader,$
                                         /tlb_size_events)

   plot_base = widget_base(state.w.xplotprofiles_base,$
                           /column,$
                           frame=2)
  
      row = widget_base(plot_base,$
                        /row)

         button = widget_button(row,$
                                font=buttonfont,$
                                event_pro = 'xplotprofiles_button_event',$
                                value='Plot PS',$
                                uvalue='Plot PS')

         button = widget_button(row,$
                                font=buttonfont,$
                                value='Write FITS',$
                                event_pro = 'xplotprofiles_button_event',$
                                uvalue='Write FITS')

         button = widget_button(row,$
                                FONT=buttonfont,$
                                VALUE='Range',$
                                event_pro = 'xplotprofiles_button_event',$
                                UVALUE='Range')
         
         quit_button = widget_button(row,$
                                     font=buttonfont,$
                                     value='Done',$
                                     uvalue='Quit',$
                                     event_pro = 'xplotprofiles_button_event')
         

         state.w.message = widget_text(row, $
                                       YSIZE=1)

      state.w.plotwin = widget_draw(plot_base,$
                                    xsize=state.p.scrollsize[0],$
                                    ysize=state.p.ysize,$
                                    x_scroll_size=state.p.scrollsize[0],$
                                    y_scroll_size=state.p.scrollsize[1],$
                                    event_pro = 'xplotprofiles_plotwin_event',$
                                    /scroll,$
                                    /motion_events,$
                                    /button_events,$
                                    uvalue='Plot Window')
      
widget_control, state.w.xplotprofiles_base, /Realize

;  Get plotwin ids

widget_control, state.w.plotwin, get_value = x
state.p.plotwin_wid = x

window, /free, /pixmap,xsize=state.p.scrollsize[0],$
  ysize=state.p.ysize
state.p.pixmap_wid = !d.window

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xplotprofiles', $
  state.w.xplotprofiles_base, $
  /No_Block,$
  event_handler='xplotprofiles_resize',$
  cleanup = 'xplotprofiles_cleanup'

geom = widget_info(state.w.xplotprofiles_base, /geometry)

state.p.buffer[0] = geom.xsize-state.p.scrollsize[0]
state.p.buffer[1] = geom.ysize-state.p.ysize

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xplotprofiles_button_event,event

common xplotprofiles_state

widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of 

    'Order': begin

        state.p.rangeorder = event.index
        xplotprofiles_minmax

    end

    'Min': begin
        
        min = cfld(state.w.min_fld,4,CANCEL=cancel)
        if cancel then goto, cont
        (*state.p.ranges)[0,state.p.rangeorder] = min
        xplotprofiles_plot
        setfocus,state.w.max_fld

    end
    'Max': begin

        max = cfld(state.w.max_fld,4,CANCEL=cancel)
        if cancel then goto, cont
        (*state.p.ranges)[1,state.p.rangeorder] = max
        xplotprofiles_plot
        setfocus,state.w.min_fld

    end

    'Plot PS': begin

        forminfo = CMPS_FORM(/INITIALIZE,SELECT='Full Portrait (color)')
        
        formInfo = CMPS_FORM(Cancel=canceled, $
                             button_names = ['Create PS File'],$
                             defaults=forminfo,$
                             Parent=state.w.xplotprofiles_base)
        
        IF NOT canceled THEN BEGIN

                thisDevice = !D.Name
                Set_Plot, "PS"
                Device, _Extra=formInfo
                xplotprofiles_plot,/PS
                Device, /Close
                Set_Plot, thisDevice

        ENDIF
        
    end

    'Quit': begin

        widget_control, event.top, /DESTROY
        !p.multi=0

    end

    'Quit Range': begin

        widget_control, event.top, /DESTROY
        state.p.fix = 0

    end

    'Range': xplotprofiles_range

    'Write FITS': begin

        path = dialog_pickfile(DIALOG_PARENT=state.w.xplotprofiles_base,$
                               file='idl.fits')
        if path ne '' then writefits,path,[[*state.d.x_arcs],$
                                           [*state.d.profiles]]

    end

endcase
cont:

end
;
;******************************************************************************
;
pro xplotprofiles_plotwin_event,event

common xplotprofiles_state

!p = state.p.pscale
!x = state.p.xscale
!y = state.p.yscale
x  = event.x/float(state.p.scrollsize[0])
y  = event.y/float(state.p.ysize)
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

;  Update cursor position.

    label = 'Cursor X: '+strtrim(xy[0],2)+'
    widget_control,state.w.message,set_value=label
    
;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin_wid
device, copy=[0,0,state.p.scrollsize[0],state.p.ysize,0,0,$
              state.p.pixmap_wid]
plots, [event.x,event.x],[0,state.p.ysize],color=2,/DEVICE


end
;
;******************************************************************************
;
pro xplotprofiles_resize, event

common xplotprofiles_state

widget_control, state.w.xplotprofiles_base, tlb_get_size = size

state.p.scrollsize[0]=size[0]-state.p.buffer[0]
state.p.scrollsize[1]=size[1]-state.p.buffer[1]

widget_control, state.w.plotwin, xsize=state.p.scrollsize[0]
widget_control, state.w.plotwin, ysize=state.p.scrollsize[1]
widget_control, state.w.plotwin, draw_xsize=state.p.scrollsize[0]

state.p.ysize = state.p.scrollsize[1] > $
  state.p.pixpp*state.d.norders*(1 > (2-state.d.norders*0.1))

widget_control, state.w.plotwin, draw_ysize=state.p.ysize
widget_control, state.w.plotwin, /UPDATE

wdelete,state.p.pixmap_wid
window, /free, /pixmap,xsize=state.p.scrollsize[0],$
  ysize=state.p.ysize
state.p.pixmap_wid = !d.window

xplotprofiles_plot

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro xplotprofiles,data,orders,doorders,slith_arc,POSITIONS=positions,$
                  MASK=mask,GROUP_LEADER=group_leader,BGFIT=bgfit,CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 4 then begin
    
    print, 'Syntax - xplotprofiles,data,orders,doorders,slith_arc,$'
    print, '                       POSITIONS=positions,MASK=mask,$'
    print, '                       GROUP_LEADER=group_leader,BGFIT=bgfit,$'
    print, '                       CANCEL=cancel '
    cancel = 1
    return

endif
zparcheck, 'xplotprofiles', data, 1, 8, [0,1], 'Data'
zparcheck, 'xplotprofiles', orders, 2, [2,3,4,5], [0,1], 'Orders'
zparcheck, 'xplotprofiles', doorders, 3, [2,3,4,5], [0,1], 'Doorders'
zparcheck, 'xplotprofiles', slith_arc, 4, [2,3,4,5], 0, 'Slith_arc'


common xplotprofiles_state

if not xregistered('xplotprofiles') then xplotprofiles_initcommon

state.d.norders = n_elements(orders)
state.p.ysize = state.p.scrollsize[1] > $
  state.p.pixpp*state.d.norders*(1 > (2-state.d.norders*0.1))
*state.d.orders   = orders
*state.p.doorders = doorders
*state.d.profiles = data
state.p.slith = slith_arc

if not xregistered('xplotprofiles') then xplotprofiles_startup,$
  GROUP_LEADER=group_leader

state.p.nodata = 0
if n_elements(POSITIONS) ne 0 then begin
    
    state.p.npeaks = (size(positions))[1]
    state.p.positions = 1
    *state.d.positions = positions
    state.p.nodata = 1

endif else state.p.positions = 0
if n_elements(MASK) ne 0 then begin
    
    state.p.mask = 1
    *state.d.mask = mask
    
endif else state.p.mask = 0
    
if n_elements(BGFIT) ne 0 then state.p.bgfit = bgfit

xplotprofiles_plot
!p.multi=0


end


