; NAME:
;     xselectspec
;
; PURPOSE:
;     To eliminate bad spectra from a stack
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xselectspec,wave,stack,mask,XUNITS=xunits,YUNITS=yunits,$
;                 GROUP_LEADER=group_leader,IMASK=imask,ALLORDERS=allorders,$
;                 yrange=yrange,CANCEL=cancel
;
; INPUTS:
;     wave  - A 1-D wavelength array
;     stack - An array [*,nspec] of spectra
;
; OUTUTS:
;     mask - An array [nspec] of 1s and 0s where good = 1 and bad = 0
;
; KEYWORD PARAMETERS:    
;     XUNITS       - A string giving the wavelength units
;     YUNITS       - A string giving the flux units 
;     GROUP_LEADER - The widget ID of an existing widget that serves
;                    as "group leader" for the newly-created widget. 
;     IMASK        - An array [nspec] or 1s and 0s giving and initial
;                    mask
;     ALLORDERS    - Set on return if the bad spectrum should be
;                    removed from all orders
;     CANCEL       - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     Alot
;
; PROCEDURE:
;     The widget is resizable.  Unselect bad spectra.  Select 
;     'All Orders' to set ALLORDERS. Used with xcombspec.pro.
;
; MODIFICATION HISTORY:
;     ? - Written by M. Cushing, Institute for Astronomy, UH
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xselectspec_initcommon,wave,stack,IMASK=imask,YRANGE=yrange

common xselectspec_state, state

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {idwin:0,$
     keyboard:0L,$
     mask_bg:0L,$
     message:0L,$
     plotwin:0,$
     xselectspec_base:0L,$
     xmin_fld:[0L,0L],$
     xmax_fld:[0L,0L],$
     ymin_fld:[0L,0L],$
     ymax_fld:[0L,0L]}

nspec = (size(stack))[2]
mask = (n_elements(IMASK) ne 0) ? imask:(intarr(nspec)+1)

r = {allorders:0,$
     colors:[1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6,7,8,9,10,$
             1,2,3,4,5,6,7,8,9,10,1,2,3,4,5,6,7,8,9,10],$
     lines:[0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,$
            3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4],$
     cancel:0,$
     cursormode:'None',$
     nspec:nspec,$
     plotmask:intarr(nspec)+1,$
     selectmask:mask}

d = {oflux:stack,$
     owave:wave}

if n_elements(YRANGE) eq 0 then yrange = [0,max(stack,/NAN,min=min)]
xmax = max(wave,min=xmin)

p = {buffer:[0.,0.],$
     cursor:0,$
     idwin_wid:0L,$
     pixmap_wid:0L,$
     plotwin_wid:0L,$
     plotxrange:[xmin,xmax],$
     plotyrange:yrange,$
     plotsize:[600,450],$
     pscale:!p,$
     psinfo:CMPS_FORM(/INITIALIZE),$
     xscale:!x,$
     xunits:'',$
     yscale:!y,$
     yunits:'',$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]]}

;  Load the three structures in the state structure.

state = {w:w,r:r,d:d,p:p}

end
;
;******************************************************************************
;
pro xselectspec_plotidwin

common xselectspec_state

wset, state.p.idwin_wid

plot,indgen(state.r.nspec+2),/NODATA,$
  xrange=[0,1],yrange=[0,state.r.nspec+1],xsty=5,ysty=5,$
  xmargin=[0,0],ymargin=[0,0]

for i = 0, state.r.nspec-1 do begin

    plots,[0,0.5],[state.r.nspec-i,state.r.nspec-i],color=state.r.colors[i],$
      linestyle=state.r.lines[i]
    xyouts,0.6,state.r.nspec-i,'Spec '+string(i+1,format='(i2.2)'),$
      color=state.r.colors[i]

endfor



end
;
;******************************************************************************
;
pro xselectspec_plotspec

common xselectspec_state

z = where(state.r.selectmask eq 1,count)

wset, state.p.plotwin_wid
erase
plot, state.d.owave,state.d.oflux[*,0],/NODATA,/xsty,/ysty,$
  yrange=state.p.plotyrange,xrange=state.p.plotxrange

wset, state.p.pixmap_wid
erase
plot, state.d.owave,state.d.oflux[*,0],/NODATA,/xsty,/ysty,$
  yrange=state.p.plotyrange,xrange=state.p.plotxrange
state.p.pscale = !p
state.p.xscale = !x
state.p.yscale = !y

for i = 0, state.r.nspec-1 do begin

    if state.r.selectmask[i] eq 0 then goto, cont
    
    wset, state.p.plotwin_wid
    oplot,state.d.owave,state.d.oflux[*,i],color=state.r.colors[i],$
      linestyle=state.r.lines[i],psym=10

    wset, state.p.pixmap_wid
    oplot,state.d.owave,state.d.oflux[*,i],color=state.r.colors[i],$
      linestyle=state.r.lines[i],psym=10


    cont:

endfor

;set_plot,'ps'
;device, filename='spec.ps',/landscape,/color
;plot, state.d.owave,state.d.oflux[*,0],/NODATA,/xsty,/ysty,$
;  yrange=state.p.plotyrange,xrange=state.p.plotxrange

;for i = 0, state.r.nspec-1 do begin

;    if state.r.selectmask[i] eq 0 then goto, cont1
;    color = state.r.colors[i]
;    if color eq 1 then color=0
;    if color eq 5 then color=7
;    oplot,state.d.owave,state.d.oflux[*,i],color=color,$
;      linestyle=state.r.lines[i],psym=10
;    cont1:

;endfor

;device, /close
;set_plot,'x'


end
;
;******************************************************************************
;
pro xselectspec_setminmax

common xselectspec_state

widget_control, state.w.xmin_fld[1],set_value=strtrim(state.p.plotxrange[0],2)
widget_control, state.w.xmax_fld[1],set_value=strtrim(state.p.plotxrange[1],2)
widget_control, state.w.ymin_fld[1],set_value=strtrim(state.p.plotyrange[0],2)
widget_control, state.w.ymax_fld[1],set_value=strtrim(state.p.plotyrange[1],2)

end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xselectspec_event,event

common xselectspec_state

widget_control, event.id,  GET_UVALUE = uvalue

case uvalue of

    'Accept': widget_control, event.top, /DESTROY

    'All Orders': state.r.allorders = event.select


    'Cancel': begin

        state.r.cancel = 1
        widget_control, event.top, /DESTROY

    end

    'Keyboard': begin

        case strtrim(event.ch,2) of 

            'c': begin ; Clear

                state.r.cursormode = 'None'
                state.p.reg = [[!values.f_nan,!values.f_nan],$
                               [!values.f_nan,!values.f_nan]]
                xselectspec_plotspec
                
            end

            'w': begin

                state.p.plotxrange = [min(state.d.owave,MAX=xmax),xmax]
                z = where(state.r.selectmask eq 1)
                state.p.plotyrange = [0.,max(state.d.oflux[*,z],/NAN)]
                xselectspec_plotspec
                xselectspec_setminmax
                
            end
            
            'z': begin ; Zoom

                if state.r.cursormode eq 'None' then begin

                    state.r.cursormode = 'Zoom'
                    state.p.reg = [[!values.f_nan,!values.f_nan],$
                                   [!values.f_nan,!values.f_nan]]
                    
                endif

            end
        
            else:
            
        endcase
           
    end

    'Help Done': widget_control, event.top, /DESTROY

    'Select Spectra': begin

        mask = state.r.selectmask
        mask[event.value] = event.select
        z = where(mask eq 1,count)
        if count eq 0 then begin

            ok = dialog_message('At least one spectrum must be selected.',$
                                /ERROR,DIALOG_PARENT=state.w.xselectspec_base)
            widget_control, state.w.mask_bg, SET_VALUE=state.r.selectmask


        endif else begin

            state.r.selectmask = mask
            xselectspec_plotspec

        endelse

    end
endcase

cont: 

end
;
;******************************************************************************
;
pro xselectspec_minmax_event,event

common xselectspec_state

xmin = cfld(state.w.xmin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmin2 = crange(xmin,state.p.plotxrange[1],'X Min',/KLT,$
               WIDGET_ID=state.w.xselectspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmin_fld[0],set_value=state.p.plotxrange[0]
    return

endif else state.p.plotxrange[0] = xmin2

xmax = cfld(state.w.xmax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
xmax2 = crange(xmax,state.p.plotxrange[0],'X Max',/KGT,$
               WIDGET_ID=state.w.xselectspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.xmax_fld[0],set_value=state.p.plotxrange[1]
    return

endif else state.p.plotxrange[1] = xmax2

ymin = cfld(state.w.ymin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymin2 = crange(ymin,state.p.plotyrange[1],'Y Min',/KLT,$
               WIDGET_ID=state.w.xselectspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymin_fld[0],set_value=state.p.plotyrange[0]
    return

endif else state.p.plotyrange[0] = ymin2

ymax = cfld(state.w.ymax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
ymax2 = crange(ymax,state.p.plotyrange[0],'Y Max',/KGT,$
               WIDGET_ID=state.w.xselectspec_base,CANCEL=cancel)
if cancel then begin

    widget_control, state.w.ymax_fld[0],set_value=state.p.plotyrange[1]
    return

endif else state.p.plotyrange[1] = ymax2

xselectspec_plotspec

end
;
;******************************************************************************
;
pro xselectspec_winevent,event

common xselectspec_state

widget_control, event.id,  GET_UVALUE = uvalue

;  Check to see if it is a TRACKING event.

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE

!p = state.p.pscale
!x = state.p.xscale
!y = state.p.yscale
x  = event.x/float(state.p.plotsize[0])
y  = event.y/float(state.p.plotsize[1])
xy = convert_coord(x,y,/NORMAL,/TO_DATA)

if event.type eq 1 then begin
    
    if state.r.cursormode eq 'None' then goto, cont
    z = where(finite(state.p.reg) eq 1,count)
    if count eq 0 then state.p.reg[*,0] = xy[0:1] else begin 
        
        state.p.reg[*,1] = xy[0:1]
        state.p.plotxrange   = [min(state.p.reg[0,*],max=max),max]
        state.p.plotyrange   = [min(state.p.reg[1,*],max=max),max]
        xselectspec_plotspec
        state.r.cursormode   = 'None'
        xselectspec_setminmax
        
    endelse

endif

;  Copy the pixmaps and draw the lines.

wset, state.p.plotwin_wid
device, copy=[0,0,state.p.plotsize[0],state.p.plotsize[1],0,0,$
              state.p.pixmap_wid]
plots, [event.x,event.x],[0,state.p.plotsize[1]],color=2,/DEVICE
plots, [0,state.p.plotsize[0]],[event.y,event.y],color=2,/DEVICE

if state.r.cursormode eq 'Zoom' then begin

    xy = convert_coord(event.x,event.y,/DEVICE,/TO_DATA)
    plots,[state.p.reg[0,0],state.p.reg[0,0]],[state.p.reg[1,0],xy[1]],$
      linestyle=2,color=2
    plots, [state.p.reg[0,0],xy[0]],[state.p.reg[1,0],state.p.reg[1,0]],$
      linestyle=2,color=2

endif
    


label = 'Cursor X: '+strtrim(xy[0],2)+', Y:'+strtrim(xy[1],2)
widget_control,state.w.message,set_value=label
    
cont:
   
end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro xselectspec,wave,stack,mask,XUNITS=xunits,YUNITS=yunits,$
                GROUP_LEADER=group_leader,IMASK=imask,ALLORDERS=allorders,$
                CANCEL=cancel,yrange=yrange

if n_params() eq 3 then begin
    
    zparcheck, 'xselectspec', wave, 1, [2,3,4,5], 1, 'Wave' 
    zparcheck, 'xselectspec', stack, 1, [2,3,4,5], [1,2], 'Stack' 

    xselectspec_initcommon,wave,stack,IMASK=imask,YRANGE=yrange

    common xselectspec_state
    
    modal = (n_elements(group_leader) ne 0) ? 1:0

;  Build the widget.

    buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'
    textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'


    state.w.xselectspec_base = widget_base(TITLE='XSelectspec', $
                                           GROUP_LEADER=group_leader,$
                                           MODAL=modal,$
                                           /COLUMN)

       cancel = widget_button(state.w.xselectspec_base,$
                              FONT=buttonfont,$
                              EVENT_PRO='xselectspec_event',$
                              VALUE='Cancel',$
                              UVALUE='Cancel')

       row_base = widget_base(state.w.xselectspec_base,$
                              /ROW)
       
          col1_base = widget_base(row_base,$
                                  EVENT_PRO='xselectspec_event',$
                                  /FRAME,$
                                  /COLUMN)

          if state.r.nspec gt 15 then begin

              value = string(findgen(state.r.nspec)+1,format='(i2.2)')
              state.w.mask_bg = cw_bgroup(col1_base,$
                                          FONT=buttonfont,$
                                          'Spec '+value,$
                                          /COLUMN,$
                                          /SCROLL,$
                                          XSIZE=100,$
                                          X_SCROLL_SIZE=100,$
                                          Y_SCROLL_SIZE=400,$
                                          YSIZE=1000,$
                                          /NONEXCLUSIVE,$
                                          LABEL_TOP='Select Spectra:',$
                                          UVALUE='Select Spectra',$
                                          SET_VALUE=state.r.selectmask)
              
              
          endif else begin

              value = string(findgen(state.r.nspec)+1,format='(i2.2)')
              state.w.mask_bg = cw_bgroup(col1_base,$
                                          FONT=buttonfont,$
                                          'Spec '+value,$
                                          /COLUMN,$
                                          /NONEXCLUSIVE,$
                                          LABEL_TOP='Select:',$
                                          UVALUE='Select Spectra',$
                                          SET_VALUE=state.r.selectmask)

          endelse

;          all_bg = cw_bgroup(col1_base,$
;                             ['All Orders'],$
;                             FONT=buttonfont,$
;                             UVALUE='All Orders',$
;                             /NONEXCLUSIVE)

             accept = widget_button(col1_base,$
                                    FONT=buttonfont,$
                                    VALUE='Accept',$
                                    UVALUE='Accept')

         col2_base = widget_base(row_base,$
                                  /FRAME,$
                                  /COLUMN)
             
             state.w.message = widget_text(col2_base, $
                                           YSIZE=1)

             state.w.keyboard = widget_text(col2_base, $
                                            /ALL_EVENTS,$
                                            SCR_XSIZE=1, $
                                            SCR_YSIZE=1, $
                                            EVENT_PRO='xselectspec_event',$
                                            UVALUE='Keyboard',$
                                            VALUE='')
                   
             row = widget_base(col2_base,$
                               /ROW)

                state.w.idwin = widget_draw(row,$
                                            XSIZE=150,$
                                            YSIZE=state.p.plotsize[1],$
                                            UVALUE='ID Window')

             
                state.w.plotwin = widget_draw(row,$
                                              XSIZE=state.p.plotsize[0],$
                                              YSIZE=state.p.plotsize[1],$
                                              /TRACKING_EVENTS,$
                                              /BUTTON_EVENTS,$
                                              /MOTION_EVENTS,$
                                             EVENT_PRO='xselectspec_winevent',$
                                              UVALUE='Plot Window 1')

             row_base = widget_base(col2_base,$
                               /FRAME,$
                               /ROW)
   
                xmin = coyote_field2(row_base,$
                                     labelfont=buttonfont,$
                                     fieldfont=textfont,$
                                     title='X Min:',$
                                     uval = 'X Min',$
                                     xsize=12,$
                                     event_pro = 'xselectspec_minmax_event',$
                                     /cr_only,$
                                     textid=textid)
                state.w.xmin_fld = [xmin,textid]
                
                xmax = coyote_field2(row_base,$
                                     labelfont=buttonfont,$
                                     fieldfont=textfont,$
                                     title='X Max:',$
                                     uval = 'X Max',$
                                     xsize=12,$
                                     event_pro = 'xselectspec_minmax_event',$
                                     /cr_only,$
                                     textid=textid)
                state.w.xmax_fld = [xmax,textid]
                
                ymin = coyote_field2(row_base,$
                                     labelfont=buttonfont,$
                                     fieldfont=textfont,$
                                     title='Y Min:',$
                                     uval = 'Y Min',$
                                     xsize=12,$
                                     event_pro = 'xselectspec_minmax_event',$
                                     /cr_only,$
                                     textid=textid)
                state.w.ymin_fld = [ymin,textid]
                
                ymax = coyote_field2(row_base,$
                                     labelfont=buttonfont,$
                                     fieldfont=textfont,$
                                     title='Y Max:',$
                                     uval = 'Y Max',$
                                     xsize=12,$
                                     event_pro = 'xselectspec_minmax_event',$
                                     /cr_only,$
                                     textid=textid)
                state.w.ymax_fld = [ymax,textid]
                
             
; Get things running.  Center the widget using the Fanning routine.
          
    centertlb,state.w.xselectspec_base
    widget_control, state.w.xselectspec_base, /REALIZE

;  Get plotwin ids
          
    widget_control, state.w.plotwin, get_value = x
    state.p.plotwin_wid = x

    widget_control, state.w.idwin, get_value = x
    state.p.idwin_wid = x

    window, /free, /pixmap,xsize=state.p.plotsize[0],$
      ysize=state.p.plotsize[1]
    state.p.pixmap_wid = !d.window

    mkct
    xselectspec_plotidwin
    xselectspec_plotspec
    xselectspec_setminmax


; Start the Event Loop. This will be a non-blocking program.
    
    XManager, 'xselectspec', $
      state.w.xselectspec_base

    cancel = state.r.cancel
;    allorders = state.r.allorders
    allorders = 1
    if cancel then begin

        mask = (n_elements(IMASK) ne 0) ? imask:intarr((size(stack))[2])+1

    endif else mask = state.r.selectmask

endif

end
