; NAME:
;     xvspec
;
; PURPOSE:
;     To display SpeX FITS spectra files
;       
; CATEGORY:
;     Data reduction
;
; CALLING SEQUENCE:
;     xvspec
;     xvspec,spectra,norders,naps,ORDERS=orders,YUNITS=yunits,XUNITS=xunits,$
;            START=start,STOP=stop,FILE=file,HDR=hdr,CANCEL=cancel
;
; INPUTS:
;     spectra - An array [*,3,naps*norders] of spectra where
;               array[*,0,0] - wavelength
;               array[*,1,0] - flux
;               array[*,2,0] - error
;     norders - The number of orders
;     naps    - The number of apertures
;
; OUTUTS:
;     None
;
; KEYWORD PARAMETERS:    
;     ORDERS       - An array [norders] of order numbers
;     XUNITS       - A string giving the wavelength units
;     YUNITS       - A string giving the flux units 
;     START        - Start column.  
;     STOP         - Stop column
;                    If START and STOP are given, then the pixel axis
;                    can be plotted as well.
;     FILE         - If given, the name of the file will be put on the
;                    title bar.
;     HDR          - A string array of the FITS header
;     CANCEL       - Set on return if there is a problem
;
; PROCEDURES CALLED:
;     Requires the Astronomy User's Library
;     Alot
;
; PROCEDURE:
;     Straight forward
;
; MODIFICATION HISTORY:
;     ? - Written by M. Cushing, Institute for Astronomy, UH
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xvspec_initcommon

;  Build the structures which will hold the important info.
;  w - contains info pertaining to widget operations.

common xvspec_state, state

w = {aperture_dl:0,$
     fix_bg:0L,$
     hdr_base:0L,$
     hdr_text:0L,$
     header:0,$
     max_fld:[0,0],$
     min_fld:[0,0],$
     order_dl:0L,$
     plotwin:0L,$
     range_base:0L,$
     xvspec_base:0L}

p = {ap:1,$
     buffer:[0,0],$
     charsize:2,$
     color:3,$
     fix:0,$
     hdr:0,$
     minpix:0,$
     maxpix:0,$
     pixelaxis:'No',$
     pixmap_wid:0L,$
     plotwin_wid:0,$
     plotwinsize:[512,500],$
     ranges:ptr_new(fltarr(2)),$
     rangeorder:0,$
     screen_size:get_screen_size( ),$
     smooth:1,$
     spectype:'Flux',$
     widgetsize:[0,0],$
     xunits:' ',$
     yunits:' '}

d = {hdr:ptr_new(strarr(2)),$
     path:'',$
     orders:ptr_new([1,2,3]),$
     naps:0L,$
     norders:0L,$
     spectra:ptr_new(fltarr(2))}

state = {w:w,p:p,d:d}

end
;
;******************************************************************************
;
pro xvspec_ap

common xvspec_state

value = 'Aperture '+strcompress(indgen(state.d.naps)+1, /re)

widget_control, state.w.aperture_dl, set_value=value

end
;
;******************************************************************************
;
pro xvspec_cleanup,event

common xvspec_state

ptr_free, state.p.ranges
ptr_free, state.d.hdr
ptr_free, state.d.orders
ptr_free, state.d.spectra

state = 0B

end
;
;******************************************************************************
;
pro xvspec_hdr

common xvspec_state
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not (xregistered('xvspec_hdrinfo')) then begin

    state.w.hdr_base = widget_base(group_leader=state.w.xvspec_base, $
                                   TITLE='FITS Header',$
                                   /COLUMN, $
                                   UVALUE='headinfo_base')

    state.w.hdr_text = widget_text(state.w.hdr_base, $
                                   /scroll, $
                                   font='7x14',$
                                   value = (*state.d.hdr), $
                                   xsize = 85, $
                                   ysize = 24)
    
    button = widget_button(state.w.hdr_base, $
                           value = 'Done',$
                           font=buttonfont,$
                           event_pro = 'xvspec_quit')
    
    widget_control, state.w.hdr_base, /realize
    XManager, 'xvspec_hdrinfo', $
      state.w.hdr_base, $
      /No_Block,$
      event_handler='xvspec_resize'

endif else begin

    widget_control, state.w.hdr_text, set_value=(*state.d.hdr)

endelse

end
;
;******************************************************************************
;
pro xvspec_maxmin

common xvspec_state

widget_control, state.w.min_fld[1], set_value=strcompress($
  (*state.p.ranges)[0,state.p.rangeorder], /re)

widget_control, state.w.max_fld[1], set_value=strcompress($
  (*state.p.ranges)[1,state.p.rangeorder], /re)

end
;
;******************************************************************************
;
pro xvspec_plot,PS=ps

common xvspec_state

if not keyword_set(PS) then begin
    
    wset, state.p.plotwin_wid
    erase
    wset, state.p.pixmap_wid
    erase

endif

!p.multi[2] = state.d.norders
!p.multi[0] = state.d.norders

idx = reverse( findgen(state.d.norders) * state.d.naps + (state.p.ap) )

if not state.p.fix then *state.p.ranges = fltarr(2,state.d.norders)

for i = 0, state.d.norders-1 do begin

    j   = state.d.norders-1-i

    title= ((*state.d.orders)[0] ne 999) ? 'Order '+$
      strcompress((*state.d.orders)[j], /re):''
    
    if state.p.smooth ne 1 then begin

        convolvespec,findgen(n_elements((*state.d.spectra)[*,1,idx[i]])),$
          (*state.d.spectra)[*,1,idx[i]],state.p.smooth,cflux,cerror,$
          ERROR=(*state.d.spectra)[*,2,idx[i]],CANCEL=cancel
        if cancel then return

    endif else begin

        cflux  = (*state.d.spectra)[*,1,idx[i]]
        cerror = (*state.d.spectra)[*,2,idx[i]]

    endelse
        
    case state.p.spectype of 

        'Flux': begin

            spec = cflux
            xtitle='!5Wavelength ('+state.p.xunits+')'
            ytitle='!5Flux ('+state.p.yunits+')'
 
        end
        'Error': begin

            spec = cerror
            xtitle='!5Wavelength ('+state.p.xunits+')'
            ytitle='!5Flux ('+state.p.yunits+')'            

        end
        'S/N': begin

            spec = cflux/cerror
            xtitle='!5Wavelength ('+state.p.xunits+')'
            ytitle='!5S/N'

        end
    endcase

    xstyle = (state.p.pixelaxis eq 'Yes') ? 9:1

    if state.p.fix then begin

        if not keyword_set(PS) then wset, state.p.plotwin_wid

        plot,(*state.d.spectra)[*,0,idx[i]],spec,$
          xsty=xstyle,xtitle=xtitle,ytitle=ytitle,$,$
          charsize=state.p.charsize,title=title,/nodata,$
          yrange=(*state.p.ranges)[*,i],/ysty,ticklen=0.04

        if not keyword_set(PS) then begin

            !p.multi[0] = J+1
            wset, state.p.pixmap_wid
            plot,(*state.d.spectra)[*,0,idx[i]],spec,$
              xsty=xstyle,xtitle=xtitle,ytitle=ytitle,$,$
              charsize=state.p.charsize,title=title,/nodata,$
              yrange=(*state.p.ranges)[*,i],/ysty,ticklen=0.04

        endif

    endif else begin

        max = max(spec,/NAN)

        if not keyword_set(PS) then wset, state.p.plotwin_wid

        plot,(*state.d.spectra)[*,0,idx[i]],spec,$
          xsty=xstyle,xtitle=xtitle,ytitle=ytitle,$
          charsize=state.p.charsize,title=title,/nodata,yrange=[0.,max],/ysty,$
          ticklen=0.04
        (*state.p.ranges)[*,i] = !y.crange

        if not keyword_set(PS) then begin

            !p.multi[0] = J+1
            wset, state.p.pixmap_wid
            plot,(*state.d.spectra)[*,0,idx[i]],spec,$
              xsty=xstyle,xtitle=xtitle,ytitle=ytitle,$
              charsize=state.p.charsize,title=title,/nodata,yrange=[0.,max],$
              /ysty,ticklen=0.04
            (*state.p.ranges)[*,i] = !y.crange
            
        endif

    endelse

    if not keyword_set(PS) then wset, state.p.plotwin_wid
    if keyword_set(PS) and state.p.color eq 1 then color=0 else $
      color=state.p.color


    oplot,(*state.d.spectra)[*,0,idx[i]],spec,color=color,psym=10

    if xstyle eq 9 then $
      axis,xaxis=1,xrange=[state.p.minpix,state.p.maxpix],/xsty,$
      charsize=state.p.charsize

    if not keyword_set(PS) then begin

        wset, state.p.pixmap_wid
        oplot,(*state.d.spectra)[*,0,idx[i]],spec,color=color,psym=10
        
        if xstyle eq 9 then $
          axis,xaxis=1,xrange=[state.p.minpix,state.p.maxpix],/xsty,$
          charsize=state.p.charsize
        
    endif

endfor
!p.multi = 0

end
;
;******************************************************************************
;
pro xvspec_ps

common xvspec_state

forminfo = CMPS_FORM(/INITIALIZE,SELECT='Full Portrait (color)')

formInfo = CMPS_FORM(Cancel=canceled, Create=create, $
                     defaults=forminfo,$
                     button_names = ['Create PS File'],$
                     Parent=state.w.xvspec_base)

   IF NOT canceled THEN BEGIN

           thisDevice = !D.Name
           Set_Plot, "PS"
           Device, _Extra=formInfo
           state.p.charsize=1
           xvspec_plot,/PS
           state.p.charsize=2
           Device, /Close
           Set_Plot, thisDevice
           !p.background=0


     ENDIF

end
;
;******************************************************************************
;
pro xvspec_range

common xvspec_state

textfont   = '-adobe-helvetica-medium-r-normal--0-0-75-75-p-0-iso8859-1'
buttonfont = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

if not xregistered('xvspec_range') then begin

    state.w.range_base = widget_base(group_leader = state.w.xvspec_base, $
                                     /column, $
                                     title ='Plot Range')

       value = 'Order '+strcompress( reverse((*state.d.orders)), /re)
       state.w.order_dl = widget_droplist(state.w.range_base,$
                                          font=buttonfont,$
                                          value=value,$
                                          /dynamic_resize,$
                                          uvalue='Order')  

       min = coyote_field2(state.w.range_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='Min:',$
                           uvalue='Min',$
                           xsize=10,$
                           /cr_only,$
                           event_pro='xvspec_buttons',$
                           textid=textid)
       state.w.min_fld = [min,textid]

       max = coyote_field2(state.w.range_base,$
                           labelfont=buttonfont,$
                           fieldfont=textfont,$
                           title='Max:',$
                           uvalue='Max',$
                           xsize=10,$
                           /cr_only,$
                           event_pro='xvspec_buttons',$
                           textid=textid)
       state.w.max_fld = [max,textid]

       kill = widget_button(state.w.range_base,$
                            value='Quit',$
                            uvalue='Quit',$
                            font=buttonfont)

       widget_control, state.w.range_base, /Realize
       
; Start the Event Loop. This will be a non-blocking program.
       
       XManager, 'xvspec_range', $
         state.w.range_base, $
         /No_Block,$
         event_handler='xvspec_buttons'

    xvspec_maxmin

endif else begin

value = 'Order '+strcompress( reverse((*state.d.orders)), /re)
widget_control, state.w.order_dl, set_value=value
state.p.rangeorder = 0
xvspec_maxmin

endelse

state.p.fix = 1
widget_control, state.w.fix_bg, set_value=1

end
;
;******************************************************************************
;
pro xvspec_readfits,fullpath

common xvspec_state

readspec,fullpath,spectra,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits
if state.d.norders ne norders then begin
    
    state.p.fix = 0
    widget_control, state.w.fix_bg, set_value=0
    
endif

s = size(spectra)
if s[2] ne 3 then begin

    nspectra = fltarr(s[1],3,s[3])+1.
    nspectra[*,0:1,*] = spectra
    spectra = nspectra

endif

z = where(finite(spectra[*,0,0]) eq 1)
spectra = spectra[z,*,*]

*state.d.hdr     = hdr
state.p.hdr      = 1
*state.d.spectra = spectra
state.d.norders  = norders
state.d.naps     = naps
*state.d.orders  = (n_elements(ORDERS) ne 0) ? orders:1
state.p.ap       = 0
state.p.xunits   = strcompress(xunits, /re)
state.p.yunits   = strcompress(yunits, /re)
state.p.minpix   = start
state.p.maxpix   = stop

xvspec_ap
wset,state.p.plotwin_wid
erase
xvspec_plot   
if xregistered('xvspec_hdrinfo') then  xvspec_hdr
if xregistered('xvspec_range') then xvspec_range

str = 'VSpec - '+strmid(fullpath,strpos(fullpath,'/',/REVERSE_S)+1)
widget_control, state.w.xvspec_base, tlb_set_title = str

end
;
;******************************************************************************
;
pro xvspec_quit,event

common xvspec_state

widget_control, event.top, /destroy

end  
;
;******************************************************************************
;
pro xvspec_startup

common xvspec_state

mkct

;  Make the widget

font = '-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1'

state.w.xvspec_base = widget_base(title='VSpec',$
                                  /column,$
                                  /tlb_size_events)

   button = widget_button(state.w.xvspec_base,$
                          FONT=font,$
                          VALUE='Done',$
                          EVENT_PRO='xvspec_buttons',$
                          UVALUE='Quit')

   row_base = widget_base(state.w.xvspec_base,$
                          /ROW,$
                          /FRAME)

   col1_base = widget_base(row_base,$
                           EVENT_PRO='xvspec_buttons',$
                           /COLUMN)
   
      box1 = widget_base(col1_base,$
                         /COLUMN,$
                         /FRAME)
      
         row = widget_base(box1,$
                           /ROW)

            button = widget_button(row,$
                                   FONT=font,$
                                   VALUE='Read Fits',$
                                   UVALUE='Read Fits')
            
            header = widget_button(row,$
                                   FONT=font,$
                                   VALUE='Header',$
                                   UVALUE='Header')

      box2 = widget_base(col1_base,$
                         /COLUMN,$
                         /FRAME)

         bg = cw_bgroup(box2,$
                        FONT=font,$
                        ['Flux','Error','S/N'],$
                        /ROW,$
                        /RETURN_NAME,$
                        /NO_RELEASE,$
                        /EXCLUSIVE,$
                        LABEL_LEFT='',$
                        UVALUE='Spectrum Type',$
                        SET_VALUE=0)

         state.w.aperture_dl = widget_droplist(box2,$
                                               font=font,$
                                               value='Aperture 1',$
                                               uvalue='Aperture')   
         
         values = ['None','2 Pixels','3 Pixels','4 Pixels','5 Pixels',$
                   '6 Pixels','7 Pixels']
         smooth = widget_droplist(box2,$
                                  FONT=font,$
                                  TITLE='Smooth: ',$
                                  VALUE=values ,$
                                  UVALUE='Smooth')         

         box3 = widget_base(col1_base,$
                            /COLUMN,$
                            /FRAME)

            desc = ['1\Plot Color','0\White','0\Red','0\Green','0\Blue',$
                    '0\Yellow','0\Magenta','2\Cyan']
            
            pdmenu = cw_pdmenu(box3,$
                               FONT=font,$
                               desc,$
                               UVALUE='Color')

            range_base = widget_base(box3,$
                                     /ROW,$
                                     /BASE_ALIGN_CENTER)
            
               button = widget_button(range_base,$
                                      FONT=font,$
                                      VALUE='Range',$
                                      UVALUE='Range')
               
               state.w.fix_bg = cw_bgroup(range_base,$
                                          ['Fix'],$
                                          FONT=font,$
                                          UVALUE='Fix',$
                                          /NONEXCLUSIVE)
                            
         pixel_bg = cw_bgroup(box3,$
                              LABEL_LEFT = 'Pixel Axis:',$
                              ['Yes','No'],$
                              FONT=font,$
                              /RETURN_NAME,$
                              /NO_RELEASE,$
                              /ROW,$
                              UVALUE='Pixel Axis',$
                              /EXCLUSIVE,$
                              SET_VALUE=1)

         button = widget_button(col1_base,$
                                FONT=font,$
                                VALUE='Write PS',$
                                UVALUE='Write PS')


         
      col2_base = widget_base(row_base,$
                              /column)
      
         state.w.plotwin = widget_draw(col2_base,$
                                       xsize=state.p.plotwinsize[0],$
                                       ysize=state.p.plotwinsize[1],$
                                       uvalue='Plot Window',$
                                       /BUTTON_EVENTS,$
                                       event_pro='xvspec_plotwinevent')
         
widget_control, state.w.xvspec_base, /Realize

;  Get plotwin ids

widget_control, state.w.plotwin, get_value = x
state.p.plotwin_wid = x
window, /free, /pixmap,xsize=state.p.plotwinsize[0],$
  ysize=state.p.plotwinsize[1]
state.p.pixmap_wid = !d.window

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xvspec', $
  state.w.xvspec_base, $
  /No_Block,$
  event_handler='xvspec_resize',$
  cleanup = 'xvspec_cleanup'

;  Get sizes of things now for resizing

widget_control, state.w.xvspec_base, tlb_get_size = result

state.p.widgetsize[0]  = result[0]
state.p.widgetsize[1]  = result[1]

widget_geom = widget_info(state.w.xvspec_base, /geometry)
window_geom = widget_info(state.w.plotwin,  /geometry)

state.p.buffer[0] = state.p.widgetsize[0] + 2.*widget_geom.margin - $
  window_geom.xsize
state.p.buffer[1] = state.p.widgetsize[1] + 2.*widget_geom.margin - $
  window_geom.ysize



end
;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xvspec_buttons,event

common xvspec_state

widget_control, event.id,  get_uvalue = uvalue
widget_control, /hourglass

case uvalue of

    'Aperture': begin

        state.p.ap = event.index
        wset,state.p.plotwin_wid
        erase
        xvspec_plot
        if xregistered('xvspec_range') then xvspec_range
        
    end

    'Color': begin

        state.p.color = event.value
        wset,state.p.plotwin_wid
        erase
        xvspec_plot

    end

    'Fix': state.p.fix = event.select

    'Header': xvspec_hdr

    'Min': begin
        
        min = cfld(state.w.min_fld,4,CANCEL=cancel)
        if cancel then goto, cont
        (*state.p.ranges)[0,state.p.rangeorder] = min
        wset, state.p.plotwin_wid
        erase
        xvspec_plot
        setfocus,state.w.max_fld

    end
    'Max': begin

        max = cfld(state.w.max_fld,4,CANCEL=cancel)
        if cancel then goto, cont
        (*state.p.ranges)[1,state.p.rangeorder] = max
        wset, state.p.plotwin_wid
        erase
        xvspec_plot
        setfocus,state.w.min_fld

    end
    'Order': begin

        state.p.rangeorder = event.index
        xvspec_maxmin
        setfocus,state.w.min_fld


    end

    'Pixel Axis': begin

        wset,state.p.plotwin_wid
        erase
        state.p.pixelaxis = event.value
        xvspec_plot

    end
    'Quit': xvspec_quit,event
    
    'Range': xvspec_range

    'Read Fits': begin

        if state.d.path eq '' then begin

            fullpath = dialog_pickfile(dialog_parent=state.w.xvspec_base,$
                                       filter='*.fits',get_path=newpath,$
                                       /must_exist)
            state.d.path = newpath
            
        endif else begin

            fullpath = dialog_pickfile(dialog_parent=state.w.xvspec_base,$
                                       filter='*.fits',path=state.d.path,$
                                       get_path=newpath,/must_exist)          
            state.d.path = newpath

        endelse
        if fullpath eq '' then goto, cont else xvspec_readfits,fullpath

        
    end
    
    'Smooth': begin

        state.p.smooth = event.index+1
        wset,state.p.plotwin_wid
        erase
        xvspec_plot

    end

    'Spectrum Type': begin

        state.p.spectype = event.value
        xvspec_plot

    end
    'Write PS': xvspec_ps

endcase

cont:
end
;
;******************************************************************************
;
pro xvspec_plotwinevent,event

common xvspec_state

;  Copy the pixmap and draw the lines.

idx = floor(event.y/float(state.p.plotwinsize[1])*state.d.norders)
idx = idx * state.d.naps + (state.p.ap)

if state.p.smooth ne 1 then begin
    
    convolvespec,findgen(n_elements((*state.d.spectra)[*,1,idx])),$
      (*state.d.spectra)[*,1,idx],state.p.smooth,cflux,cerror,$
      ERROR=(*state.d.spectra)[*,2,idx],CANCEL=cancel
    if cancel then return
    
endif else begin
    
    cflux  = (*state.d.spectra)[*,1,idx]
    cerror = (*state.d.spectra)[*,2,idx]
    
endelse

case state.p.spectype of 
    
    'Flux': begin
        
        spec = cflux
        xtitle='!5Wavelength ('+state.p.xunits+')'
        ytitle='!5Flux ('+state.p.yunits+')'
        
    end
    'Error': begin
        
        spec = cerror
        xtitle='!5Wavelength ('+state.p.xunits+')'
        ytitle='!5Flux ('+state.p.yunits+')'            
        
    end
    'S/N': begin
        
        spec = cflux/cerror
        xtitle='!5Wavelength ('+state.p.xunits+')'
        ytitle='!5S/N'
        
    end
endcase

xzoomplot,(*state.d.spectra)[*,0,idx],spec,$
  XTITLE='Wavelength ('+state.p.xunits+')',YTITLE=ytitle

end
;
;******************************************************************************
;
pro xvspec_resize, event

common xvspec_state

widget_control, state.w.xvspec_base, tlb_get_size = size

state.p.plotwinsize[0]=size[0]-state.p.buffer[0]
state.p.plotwinsize[1]=size[1]-state.p.buffer[1]

widget_control, state.w.plotwin, draw_xsize=state.p.plotwinsize[0]
widget_control, state.w.plotwin, draw_ysize=state.p.plotwinsize[1]

wset, state.p.plotwin_wid
erase

wdelete, state.p.pixmap_wid
window, /free, /pixmap,xsize=state.p.plotwinsize[0],$
  ysize=state.p.plotwinsize[1]
state.p.pixmap_wid = !d.window

xvspec_plot

end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro xvspec,spectra,norders,naps,ORDERS=orders,YUNITS=yunits,XUNITS=xunits,$
           START=start,STOP=stop,FILE=file,HDR=hdr,CANCEL=cancel

cancel = 0

common xvspec_state

if not xregistered('xvspec') then begin

    nparms = n_params()

    case nparms of 

        3: begin

            zparcheck, 'xvspec', spectra, 1, [2,3,4,5], [2,3], 'Spectra' 
            zparcheck, 'xvspec', norders, 2, [2,3,4,5], 0, 'Norders' 
            zparcheck, 'xvspec', naps, 3, [2,3,4,5], 0, 'Naps' 
            
            xvspec_initcommon
            xvspec_startup
            z = where(finite(spectra[*,0,0]) eq 1)
            *state.d.spectra = spectra[z,*,*]
            state.d.norders  = norders
            state.d.naps     = naps
            
        end
        0: begin
        
            xvspec_initcommon
            xvspec_startup
            spectra = fltarr(1024,3)
            spectra[*,0] = findgen(1024)
            spectra[*,1] = findgen(1024)
            spectra[*,2] = findgen(1024)
            *state.d.spectra = spectra
            state.d.norders  = 1
            state.d.naps     = 1
            
        end
        else: begin

            print, 'Syntax - xvspec,spectra,norders,naps,ORDERS=orders,$'
            print, '                YUNITS=yunits,XUNITS=xunits'
            cancel = 1
            return

        end

    endcase
    *state.d.hdr     = (n_elements(HDR) eq 0) ? '':hdr
    state.p.minpix   = (n_elements(START) eq 0) ? 0:start
    state.p.maxpix   = (n_elements(STOP) eq 0) ? $
      (n_elements(spectra[*,0])-1):stop

    if n_elements(FILE) ne 0 then begin

        widget_control, state.w.xvspec_base, tlb_set_title = 'VSpec - '+file

    endif

    *state.d.orders  = (n_elements(ORDERS) ne 0) ? orders:999
    state.p.yunits   = (n_elements(YUNITS) ne 0) ? $
      strcompress(yunits, /re):''
    state.p.xunits   = (n_elements(XUNITS) ne 0) ? $
      strcompress(xunits, /re):''

endif else begin


    *state.d.hdr     = (n_elements(HDR) eq 0) ? '':hdr
    state.p.minpix   = (n_elements(START) eq 0) ? 0:start
    state.p.maxpix   = (n_elements(STOP) eq 0) ? $
      (n_elements(spectra[*,0])-1):stop

    if n_elements(FILE) ne 0 then begin

        widget_control, state.w.xvspec_base, tlb_set_title = 'VSpec - '+file

    endif

    *state.d.spectra = spectra
    state.d.norders  = norders
    state.d.naps     = naps
    *state.d.orders  = (n_elements(ORDERS) ne 0) ? orders:999
    state.p.yunits   = (n_elements(YUNITS) ne 0) ? $
      strcompress(yunits, /re):''
    state.p.xunits   = (n_elements(XUNITS) ne 0) ? $
      strcompress(xunits, /re):''
    if (xregistered('xvspec_hdrinfo')) then $
      widget_control, state.w.hdr_base, /destroy

    widget_control, state.w.xvspec_base, tlb_set_title = 'VSpec'

endelse

state.p.ap = 0

xvspec_ap
wset,state.p.plotwin_wid
erase
xvspec_plot

end
